#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
from pylab import *
from numpy import *
from casadi import *
from casadi.tools import *
import scipy.linalg
import numpy as np

"""
This example how an Linear Quadratic Regulator (LQR) can be designed
and simulated for a linear-time-invariant (LTI) system
"""

# Problem statement
# -----------------------------------

# System dimensions
ns = 3  # number of states
nu = 2  # number of controls
ny = 2  # number of outputs
N  = 21 # number of control intervals
te = 10 # end time [s]

# The system equations:   x' = A.x + B.u
A = DM([[0.4,0.1,-2],[0,-0.3,4],[1,0,0]])
B = DM([[1,1],[0,1],[1,0]])

# Inspect the open-loop system
[D,V] = linalg.eig(A)
print('Open-loop eigenvalues: ', D)

tn = linspace(0,te,N)

# Check if the system is controllable
# -----------------------------------
R = []

p = B
for i in range(ns):
  R.append(p)
  p = mtimes([A,p])

R = horzcat(*R)

# R must be of full rank
_, s, _ = linalg.svd(R)
eps = 1e-9
rank =  len([x for x in s if abs(x) > eps])
assert(rank==ns)

# Simulation of the open-loop system
# -----------------------------------

y  = SX.sym('y',ns)
u  = SX.sym('u',nu)

x0 = DM([1,0,0])
# no control
u_ = repmat(DM([[ -1, 1 ],[1,-1]]),1,int((N-1)/2))

print(u_)

p = SX.sym('p')

# tn = np.linspace(0,te,N)
# cdae = SX.fun('cdae', controldaeIn(x=y,u=u),[mtimes(A,y)+mtimes(B,u)])

# opts = {}
# opts['integrator'] = 'cvodes'
# opts['integrator_options'] = {'fsens_err_con': True,'reltol':1e-12}
# opts['nf'] = 20

# sim = ControlSimulator('sim', cdae, tn, opts)
# sim.setInput(x0,'x0')
# sim.setInput(u_,'u')
# sim.evaluate()

# tf = sim.minorT()

# figure(1)
# plot(tf,sim.getOutput().T)
# legend(('s1', 's2','s3'))
# title('reference simulation, open-loop, zero controls')
# out = sim.getOutput()

# # Simulation of the open-loop system
# #   sensitivity for initial conditions
# # -----------------------------------

# x0_pert = DM([0,0,1])*1e-4
# sim.setInput(x0+x0_pert,'x0')
# sim.setInput(u_,'u')
# sim.evaluate()

# tf = list(sim.minorT())

# figure(2)
# title('Deviation from reference simulation, with perturbed initial condition')
# plot(tf,sim.getOutput().T-out.T,linewidth=3)

# # Not supported in current revision, cf. #929
# # jacsim = sim.jacobian_old(CONTROLSIMULATOR_X0,0)
# # jacsim.setInput(x0,'x0')
# # jacsim.setInput(u_,'u')

# # jacsim.evaluate()

# # dev_est = []
# # for i in range(len(tf)):
# #   dev_est.append(mtimes(jacsim.getOutput()[i*ns:(i+1)*ns,:],x0_pert))

# # dev_est = horzcat(dev_est).T
# # plot(tf,dev_est,'+k')
# # legend(('s1 dev', 's2 dev','s3 dev','s1 dev (est.)', 's2 dev (est.)','s3 dev (est.)'),loc='upper left')


# # M = jacsim.getOutput()[-ns:,:]
# # # In the case of zero input, we could also use the matrix exponential to obtain sensitivity
# # Mref = scipy.linalg.expm(A*te)

# # e = fabs(M - Mref)
# # e = max(e)/max(fabs(M))
# # print e
# # assert(e<1e-6)


# # Simulation of the open-loop system
# #   sensitivity  for controls
# # -----------------------------------
# # What if we perturb the input?

# u_perturb = DM(u_)
# u_perturb[0,N/5] = 1e-4
# sim.setInput(x0,'x0')
# sim.setInput(u_+u_perturb,'u')
# sim.evaluate()

# figure(3)
# title('Deviation from reference simulation, with perturbed controls')
# plot(tf,sim.getOutput().T-out.T,linewidth=3)


# # Not supported in current revision, cf. #929
# # jacsim = sim.jacobian_old(CONTROLSIMULATOR_U,0)
# # jacsim.setInput(x0,'x0')
# # jacsim.setInput(u_,'u')

# # jacsim.evaluate()

# # dev_est = []
# # for i in range(len(tf)):
# #   dev_est.append(mtimes(jacsim.getOutput()[i*ns:(i+1)*ns,:],vec(u_perturb)))

# # dev_est = horzcat(dev_est).T
# # plot(tf,dev_est,'+k')
# # legend(('s1 dev', 's2 dev','s3 dev','s1 dev (est.)', 's2 dev (est.)','s3 dev (est.)'),loc='upper left')


# # Find feedforward controls
# # -----------------------------------
# #
# #  Our goal is to reach a particular end-state xref_e
# #  We can find the necessary controls explicitly in continuous time
# #  with the controllability Gramian
# #
# #  http://www-control.eng.cam.ac.uk/jmm/3f2/handout4.pdf
# #  http://www.ece.rutgers.edu/~gajic/psfiles/chap5.pdf

# x0 = vertcat([1,0,0])
# xref_e = vertcat([1,0,0])

# states = struct_symSX([
#            entry('eAt',shape=(ns,ns)),
#            entry('Wt',shape=(ns,ns))
#          ])

# eAt = states['eAt']
# Wt  = states['Wt']

# # We will find the control that allow to reach xref_e at t1
# t1 = te

# # Initial conditions
# e = densify(DM.eye(ns))
# states_ = states(0)
# states_['eAt'] = e
# states_['Wt'] = 0

# rhs = struct_SX(states)
# rhs['eAt'] = mtimes(A,eAt)
# rhs['Wt']  = mtimes([eAt,B,B.T,eAt.T])

# dae = SX.fun('dae', daeIn(x=states),daeOut(ode=rhs))

# integrator = Integrator('integrator', 'cvodes', dae, {'tf':t1, 'reltol':1e-12})
# integrator.setInput(states_,'x0')
# integrator.evaluate()

# out = states(integrator.getOutput())

# Wt_  = out['Wt']
# eAt_ = out['eAt']

# # Check that eAt is indeed the matrix exponential
# e = max(fabs(eAt_-scipy.linalg.expm(numpy.array(A*te))))/max(fabs(eAt_))
# assert(e<1e-7)

# # Simulate with feedforward controls
# # -----------------------------------

# states = struct_symSX([
#           entry('y',shape=ns),      # The regular states of the LTI system
#           entry('eAt',shape=(ns,ns))  # The matrix exponential exp(A*(t1-t))
#          ])

# eAt = states['eAt']
# y   = states['y']

# # Initial conditions
# states_ = states(0)
# states_['y'] = x0
# states_['eAt'] = eAt_

# u = mtimes([B.T,eAt.T,inv(Wt_),xref_e-mtimes(eAt_,x0)])

# rhs = struct_SX(states)
# rhs['y']   = mtimes(A,y)+mtimes(B,u)
# rhs['eAt'] = -mtimes(A,eAt)

# cdae = SX.fun('cdae', controldaeIn(x=states),[rhs])

# # Output function
# out = SX.fun('out', controldaeIn(x=states),[states,u])

# opts = {}
# opts['integrator'] = 'cvodes'
# opts['integrator_options'] = {'fsens_err_con': True,'reltol':1e-12}
# opts['nf'] = 20

# sim = ControlSimulator('sim', cdae, out, tn, opts)
# sim.setInput(states_,'x0')
# sim.evaluate()

# e = sim.getOutput()[states.i['y'],-1] - xref_e
# assert(max(fabs(e))/max(fabs(xref_e))<1e-6)

# tf = sim.minorT()
# # This will be our reference trajectory for tracking

# figure(4)
# subplot(211)
# title('Feedforward control, states')
# plot(tf,sim.getOutput(0)[list(states.i['y']),:].T)
# for i,c in enumerate(['b','g','r']):
#   plot(t1,xref_e[i],c+'o')
# subplot(212)
# title('Control action')
# plot(tf,sim.getOutput(1).T)

# # Design an infinite horizon LQR
# # -----------------------------------

# # Weights for the infinite horizon LQR control
# Q = DM.eye(ns)
# R = DM.eye(nu)

# # Continuous Riccati equation
# P = SX.sym('P',ns,ns)

# ric = (Q + mtimes(A.T,P) + mtimes(P,A) - mtimes([P,B,inv(R),B.T,P]))

# dae = SX.fun('dae', daeIn(x=vec(P)),daeOut(ode=vec(ric)))

# # We solve the ricatti equation by simulating backwards in time until steady state is reached.
# opts = {'reltol':1e-16, 'stop_at_end':False, 'tf':1}
# integrator = Integrator('integrator', 'cvodes', dae, opts)

# # Start from P = identity matrix
# u = densify(DM.eye(ns))
# xe = vec(u)

# # Keep integrating until steady state is reached
# for i in range(1,40):
#   x0 = xe
#   xe = integrator({'x0':x0})['xf']
#   e = max(fabs(xe-x0))
#   print 'it. %02d - deviation from steady state: %.2e' % (i, e)
#   if e < 1e-11:
#     break

# # Obtain the solution of the ricatti equation
# P_ = xe.reshape((ns,ns))
# print 'P=', P_

# [D,V] = linalg.eig(P_)
# assert (min(real(D))>0)
# print '(positive definite)'


# # Check that it does indeed satisfy the ricatti equation
# dae.setInput(integrator.getOutput(),'x')
# dae.evaluate()
# print max(fabs(dae.getOutput()))
# assert(max(fabs(dae.getOutput()))<1e-8)

# # From P, obtain a feedback matrix K
# K = mtimes([inv(R),B.T,P_])

# print 'feedback matrix= ', K

# # Inspect closed-loop eigenvalues
# [D,V] = linalg.eig(A-mtimes(B,K))
# print 'Open-loop eigenvalues: ', D

# # Check what happens if we integrate the Riccati equation forward in time
# dae = SX.fun('dae', daeIn(x = vec(P)),daeOut(ode=vec(-ric)))

# integrator = Integrator('integrator', 'cvodes', dae, {'reltol':1e-16, 'stop_at_end':False, 'tf':0.4})
# x0_pert = vec(P_)
# x0_pert[0] += 1e-9 # Put a tiny perturbation
# xe = x0_pert

# for i in range(1,10):
#   x0 = xe
#   xe = integrator({'x0':x0})['xf']
#   e = max(fabs(xe-x0))
#   print 'Forward riccati simulation %d; error: %.2e' % (i, e)

# # We notice divergence. Why?
# stabric = SX.fun('stabric', [P],[jacobian(-ric,P)])
# stabric.setInput(P_)
# stabric.evaluate()

# S = stabric.getOutput()

# [D,V] = linalg.eig(S)

# print 'Forward riccati eigenvalues = ', D


# # Simulation of the closed-loop system:
# #  continuous control action, various continuous references
# # ---------------------------------------------------------
# #

# x0 = DM([1,0,0])

# y  = SX.sym('y',ns)

# C = DM([[1,0,0],[0,1,0]])
# D = DM([[0,0],[0,0]])

# temp = inv(blockcat([[A,B],[C,D]]))

# F = temp[:ns,-ny:]
# Nm = temp[ns:,-ny:]

# t = SX.sym('t')

# figure(6)

# for k,yref in enumerate([ vertcat([-1,sqrt(t)]) , vertcat([-1,-0.5]), vertcat([-1,sin(t)])]):
#   u = -mtimes(K,y) + mtimes(mtimes(K,F)+Nm,yref)
#   rhs = mtimes(A,y)+mtimes(B,u)
#   cdae = SX.fun('cdae', controldaeIn(t=t, x=y),[rhs])

#   # Output function
#   out = SX.fun('out', controldaeIn(t=t, x=y),[y,mtimes(C,y),u,yref])

#   opts = {}
#   opts['integrator'] = 'cvodes'
#   opts['integrator_options'] = {'fsens_err_con': True,'reltol':1e-12}
#   opts['nf'] = 200
#   sim = ControlSimulator('sim', cdae, out, tn, opts)
#   sim.setInput(x0,'x0')
#   #sim.setInput(yref_,'u')
#   sim.evaluate()

#   tf = sim.minorT()

#   subplot(3,3,1+k*3)
#   plot(tf,sim.getOutput(0).T)
#   subplot(3,3,2+k*3)
#   title('ref ' + str(yref))
#   for i,c in enumerate(['b','g']):
#     plot(tf,sim.getOutput(1)[i,:].T,c,linewidth=2)
#     plot(tf,sim.getOutput(3)[i,:].T,c+'-')
#   subplot(3,3,3+k*3)
#   plot(tf,sim.getOutput(2).T)


# # Simulation of the closed-loop system:
# #  continuous control action, continuous feedforward reference
# # -----------------------------------------------------------
# #  To obtain a continous tracking reference,
# #  we augment statespace to construct it on the fly

# x0 = vertcat([1,0,0])

# # Now simulate with open-loop controls
# states = struct_symSX([
#            entry('y',shape=ns), # The regular states of the LTI system
#            entry('yref',shape=ns), # States that constitute a tracking reference for the LTI system
#            entry('eAt',shape=(ns,ns)) # The matrix exponential exp(A*(t1-t))
#          ])

# y     = states['y']
# eAt   = states['eAt']

# # Initial conditions
# states_ = states(0)
# states_['y']    = 2*x0
# states_['yref'] = x0
# states_['eAt']  = eAt_


# param = struct_symSX([entry('K',shape=(nu,ns))])

# param_ = param(0)

# uref = mtimes([B.T,eAt.T,inv(Wt_),xref_e-mtimes(eAt_,x0)])
# u    = uref - mtimes(param['K'],y-states['yref'])

# rhs = struct_SX(states)
# rhs['y']      =  mtimes(A,y)+mtimes(B,u)
# rhs['yref']   =  mtimes(A,states['yref'])+mtimes(B,uref)
# rhs['eAt']    = -mtimes(A,eAt)

# cdae = SX.fun('cdae', controldaeIn(x=states, p=param),[rhs])

# # Output function
# out = SX.fun('out', controldaeIn(x=states, p=param),[states,u,uref,states['yref']])

# opts = {}
# opts['integrator'] = 'cvodes'
# opts['integrator_options'] = {'fsens_err_con': True,'reltol':1e-8}
# opts['nf'] = 20
# sim = ControlSimulator('sim', cdae, out, tn, opts)

# # Not supported in current revision, cf. #929
# # jacsim = sim.jacobian_old(CONTROLSIMULATOR_X0,0)

# figure(7)

# for k,(caption,K_) in enumerate([('K: zero',DM.zeros((nu,ns))),('K: LQR',K)]):
#   param_['K'] = K_

#   sim.setInput(states_,'x0')
#   sim.setInput(param_,'p')
#   sim.evaluate()
#   sim.getOutput()

#   tf = sim.minorT()

#   subplot(2,2,2*k+1)
#   title('states (%s)' % caption)
#   for i,c in enumerate(['b','g','r']):
#     plot(tf,sim.getOutput()[states.i['yref',i],:].T,c+'--')
#     plot(tf,sim.getOutput()[states.i['y',i],:].T,c,linewidth=2)
#   subplot(2,2,2*k+2)
#   for i,c in enumerate(['b','g']):
#     plot(tf,sim.getOutput(1)[i,:].T,c,linewidth=2)
#     plot(tf,sim.getOutput(2)[i,:].T,c+'--')
#   title('controls (%s)' % caption)

#   # Not supported in current revision, cf. #929
#   # # Calculate monodromy matrix
#   # jacsim.setInput(states_,'x0')
#   # jacsim.setInput(param_,'p')
#   # jacsim.evaluate()
#   # M = jacsim.getOutput()[-states.size:,:][list(states.i['y']),list(states.i['y'])]

#   # # Inspect the eigenvalues of M
#   # [D,V] = linalg.eig(M)
#   # print 'Spectral radius of monodromy (%s): ' % caption

#   # print max(abs(D))

# print 'Spectral radius of exp((A-BK)*te): '

# [D,V] = linalg.eig(scipy.linalg.expm(numpy.array((A-mtimes(B,K))*te)))
# print max(abs(D))

# # Simulation of the controller:
# # discrete reference, continuous control action
# # -----------------------------------------------------------

# # Get discrete reference from previous simulation
# mi = sim.getMajorIndex()
# controls_ = sim.getOutput(2)[:,mi[:-1]]
# yref_     = sim.getOutput(3)[:,mi[:-1]]

# u_ = vertcat([controls_,yref_])

# x0 = DM([1,0,0])

# controls = struct_symSX([
#              entry('uref',shape=nu),
#              entry('yref',shape=ns)
#            ])

# yref  = SX.sym('yref',ns)
# y     = SX.sym('y',ns)
# dy    = SX.sym('dy',ns)
# u     = controls['uref']-mtimes(param['K'],y-controls['yref'])
# rhs   = mtimes(A,y)+mtimes(B,u)

# cdae = SX.fun('cdae', controldaeIn(x=y, u=controls, p=param),[rhs])

# # Output function
# out = SX.fun('out', controldaeIn(x=y, u=controls, p=param),[y,u,controls['uref'],controls['yref']])

# opts = {}
# opts['integrator'] = 'cvodes'
# opts['integrator_options'] = {'fsens_err_con': True,'reltol':1e-8}
# opts['nf'] = 20
# sim = ControlSimulator('sim', cdae, out, tn, opts)
# sim.setInput(2*x0,'x0')
# sim.setInput(param_,'p')
# sim.setInput(u_,'u')
# sim.evaluate()

# tf = sim.minorT()

# figure(8)
# subplot(2,1,1)
# title('states (%s)' % caption)
# for i,c in enumerate(['b','g','r']):
#   plot(tf,sim.getOutput(3)[i,:].T,c+'--')
#   plot(tf,sim.getOutput()[i,:].T,c,linewidth=2)
# subplot(2,1,2)
# for i,c in enumerate(['b','g']):
#   plot(tf,sim.getOutput(1)[i,:].T,c,linewidth=2)
#   plot(tf,sim.getOutput(2)[i,:].T,c+'--')
# title('controls (%s)' % caption)

# # Not supported in current revision, cf. #929
# # jacsim = sim.jacobian_old(CONTROLSIMULATOR_X0,0)

# # # Calculate monodromy matrix
# # jacsim.setInput(x0,'x0')
# # jacsim.setInput(param_,'p')
# # jacsim.setInput(u_,'u')
# # jacsim.evaluate()
# # M = jacsim.getOutput()[-ns:,:]

# # # Inspect the eigenvalues of M
# # [D,V] = linalg.eig(M)
# # print 'Spectral radius of monodromy, discrete reference, continous control'

# # print max(abs(D))

# # Simulation of the controller:
# # discrete reference, discrete control action
# # -----------------------------------------------------------

# y0     = SX.sym('y0',ns)

# u     = controls['uref']-mtimes(param['K'],y0-controls['yref'])
# rhs   = mtimes(A,y)+mtimes(B,u)

# cdae = SX.fun('cdae', controldaeIn(x=y, x_major=y0, u=controls, p=param),[rhs])

# # Output function
# out = SX.fun('out', controldaeIn(x=y, x_major=y0, u=controls, p=param),[y,u,controls['uref'],controls['yref']])

# opts = {}
# opts['integrator'] = 'cvodes'
# opts['integrator_options'] = {'fsens_err_con': True,'reltol':1e-8}
# opts['nf'] = 20

# sim = ControlSimulator('sim', cdae, out, tn, opts)
# sim.setInput(2*x0,'x0')
# sim.setInput(param_,'p')
# sim.setInput(u_,'u')
# sim.evaluate()

# tf = sim.minorT()

# figure(9)
# subplot(2,1,1)
# title('states (%s)' % caption)
# for i,c in enumerate(['b','g','r']):
#   plot(tf,sim.getOutput(3)[i,:].T,c+'--')
#   plot(tf,sim.getOutput()[i,:].T,c,linewidth=2)
# subplot(2,1,2)
# for i,c in enumerate(['b','g']):
#   plot(tf,sim.getOutput(1)[i,:].T,c,linewidth=2)
#   plot(tf,sim.getOutput(2)[i,:].T,c+'--')
# title('controls (%s)' % caption)

# # Not supported in current revision, cf. #929
# # jacsim = sim.jacobian_old(CONTROLSIMULATOR_X0,0)

# # # Calculate monodromy matrix
# # jacsim.setInput(x0,'x0')
# # jacsim.setInput(param_,'p')
# # jacsim.setInput(u_,'u')
# # jacsim.evaluate()
# # M = jacsim.getOutput()[-ns:,:]

# # # Inspect the eigenvalues of M
# # [D,V] = linalg.eig(M)
# # print 'Spectral radius of monodromy, discrete reference, discrete control'

# # print max(abs(D))

# show()



# This is an automatically generated file converting from the apm format

import casadi as cs

def hs021():
    # The optimal objective is (if given in):
    f_opt = -99.96
    x_opt = cs.DM([2,0])
    x = cs.MX.sym('x', 2)
    obj = cs.MX.zeros(1, 1)
    x0 = cs.DM.zeros(2, 1)
    lbx = -cs.inf*cs.DM.ones(2, 1)
    ubx = cs.inf*cs.DM.ones(2, 1)
    g = cs.MX.zeros(1, 1)
    lbg = -cs.inf*cs.DM.ones(1, 1)
    ubg = cs.inf*cs.DM.ones(1, 1)
    
    x0[0]= -1
    x0[1]= 1
    
    lbg[0] = 10
    g[0] = 10*x[0] - x[1]
    lbx[0] = 2
    ubx[0] =  50
    lbx[1] = -50
    ubx[1] =  50
    obj = x[0]**2/100 + x[1]**2 - 100
    
    
    f = obj
    
    return (x_opt, f_opt, x, f, g, lbg, ubg, lbx, ubx, x0)



import os
import sys
import subprocess

identity = sys.argv[1]
dir = sys.argv[2]

# Recursively look for all shared libraries in `dir`
for root, dirs, files in os.walk(dir):
    for file in files:
        if file.endswith(".dylib") or file.endswith(".mexmaci64") or file.endswith(".mexmaca64") or file.endswith(".so") or file.endswith(".mex") or os.access(file, os.X_OK):
            # Get the full path to the library
            path = os.path.join(root, file)

            # codesign the library
            subprocess.call(["codesign", "--remove-signature", path])
            subprocess.call(["codesign", "--force", "--sign", identity, path])



# This is an automatically generated file converting from the apm format

import casadi as cs

def hs033():
    # The optimal objective is (if given in):
    x_opt = cs.DM([0, cs.sqrt(2), cs.sqrt(2)])
    f_opt = cs.sqrt(2) - 6
    x = cs.MX.sym('x', 3)
    obj = cs.MX.zeros(1, 1)
    x0 = cs.DM.zeros(3, 1)
    lbx = -cs.inf*cs.DM.ones(3, 1)
    ubx = cs.inf*cs.DM.ones(3, 1)
    g = cs.MX.zeros(2, 1)
    lbg = -cs.inf*cs.DM.ones(2, 1)
    ubg = cs.inf*cs.DM.ones(2, 1)
    
    x0[0]= 0
    lbx[0]=0
    x0[1]= 0
    lbx[1]=0
    x0[2]= 3
    lbx[2]=0
    
    ubg[0] = 0
    g[0] = x[0]**2 + x[1]**2  - x[2]**2
    lbg[1] = 4
    g[1] = x[0]**2 + x[1]**2 + x[2]**2
    ubx[2] =  5
    obj = (x[0] - 1)*(x[0] - 2)*(x[0] - 3) + x[2]
    
    
    f = obj
    
    return (x_opt, f_opt, x, f, g, lbg, ubg, lbx, ubx, x0)



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
"""
Grabs the pdf-compiled examples and puts them in the API tree.
"""

src = 'examples/'
xml = 'XML_internal/index.xml'

import re
import shutil
from lxml import etree
import os

xmlData = etree.parse(xml)

examples = xmlData.findall("//compound[@kind='example']")


for example in examples:
  refid = example.attrib['refid']
  path = '/'.join(refid.split("/")[0:2])+'/'
  example.attrib['refid']
  name=example.findtext("name")
  m=re.search('^(.*)\.py$',name)
  if m:
    try:
      os.makedirs(os.path.dirname('html/'+m.group(1) + '.pdf'))
      os.makedirs(os.path.dirname('html/'+m.group(1) + '.py'))
    except:
      pass
    shutil.copyfile(src+m.group(1) + '.pdf','html/'+m.group(1) + '.pdf')
    shutil.copyfile(src+m.group(1) + '.py', 'html/'+m.group(1) + '.py')
    print(m.group(1) + '.pdf')



import os
import sys
import subprocess
import shutil

identity = sys.argv[1]
archive = sys.argv[2]
apple_id = sys.argv[3]
apple_team_id = sys.argv[4]
notarize_password = sys.argv[5]

# Unzip into dir
dir = "certify_dir"
notarize_dir = "notarize_dir"
shutil.rmtree(notarize_dir,ignore_errors=True)
os.mkdir(notarize_dir)
shutil.rmtree(dir,ignore_errors=True)
assert subprocess.call(["unzip",archive,"-d",dir])==0


def each_signable(dir):
    for root, dirs, files in os.walk(dir):
        for file in files:
            if file.endswith(".dylib") or file.endswith(".mexmaci64") or file.endswith(".mexmaca64") or file.endswith(".so") or file.endswith(".mex") or os.access(file, os.X_OK):
                yield os.path.join(root, file)
            

# Recursively look for all shared libraries in `dir`
for path in each_signable(dir):
    print("Signing",path)
    assert subprocess.call(["codesign", "--remove-signature", path])==0
    assert subprocess.call(["codesign", "--force", "--sign", identity, path])==0
    shutil.copy(path,notarize_dir)


assert subprocess.call(["zip","-rq","../" + "notarize_" + archive, "."],cwd=notarize_dir)==0

assert subprocess.call(["xcrun","notarytool","submit","notarize_" + archive, "--apple-id", apple_id, "--team-id", apple_team_id, "--password", notarize_password, "--wait"])==0
#for path in each_signable(dir):
#    assert subprocess.call(["xcrun", "stapler", "staple", path])==0

# Zip again
os.remove(archive)
subprocess.call(["zip","-rq","../" + archive, "."],cwd=dir)

shutil.rmtree(dir)
shutil.rmtree(notarize_dir)




#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
# -*- coding: utf-8 -*-
from casadi import *
import numpy as N
import matplotlib.pyplot as plt

'''
Demonstration on how to construct a fixed-step implicit Runge-Kutta integrator
@author: Joel Andersson, KU Leuven 2013
'''

# End time
tf = 10.0

# Dimensions
nx = 3
np = 1

# Declare variables
x  = SX.sym('x', nx)  # state
p  = SX.sym('u', np)  # control

# ODE right hand side function
ode = vertcat((1 - x[1]*x[1])*x[0] - x[1] + p, \
              x[0], \
              x[0]*x[0] + x[1]*x[1] + p*p)
dae = {'x':x, 'p':p, 'ode':ode}
f = Function('f', [x, p], [ode])

# Number of finite elements
n = 100

# Size of the finite elements
h = tf/n

# Degree of interpolating polynomial
d = 4

# Choose collocation points
tau_root = [0] + collocation_points(d, 'legendre')

# Coefficients of the collocation equation
C = N.zeros((d+1,d+1))

# Coefficients of the continuity equation
D = N.zeros(d+1)

# Dimensionless time inside one control interval
tau = SX.sym('tau')

# For all collocation points
for j in range(d+1):
  # Construct Lagrange polynomials to get the polynomial basis at the collocation point
  L = 1
  for r in range(d+1):
    if r != j:
      L *= (tau-tau_root[r])/(tau_root[j]-tau_root[r])

  # Evaluate the polynomial at the final time to get the coefficients of the continuity equation
  lfcn = Function('lfcn', [tau], [L])
  D[j] = lfcn(1.0)

  # Evaluate the time derivative of the polynomial at all collocation points to get the coefficients of the continuity equation
  tfcn = Function('tfcn', [tau], [tangent(L,tau)])
  for r in range(d+1): C[j,r] = tfcn(tau_root[r])

# Total number of variables for one finite element
X0 = MX.sym('X0',nx)
P  = MX.sym('P',np)
V = MX.sym('V',d*nx)

# Get the state at each collocation point
X = [X0] + vertsplit(V,[r*nx for r in range(d+1)])

# Get the collocation quations (that define V)
V_eq = []
for j in range(1,d+1):
  # Expression for the state derivative at the collocation point
  xp_j = 0
  for r in range (d+1):
    xp_j += C[r,j]*X[r]

  # Append collocation equations
  f_j = f(X[j],P)
  V_eq.append(h*f_j - xp_j)

# Concatenate constraints
V_eq = vertcat(*V_eq)

# Root-finding function, implicitly defines V as a function of X0 and P
vfcn = Function('vfcn', [V, X0, P], [V_eq])

# Convert to SX to decrease overhead
vfcn_sx = vfcn.expand()

# Create a implicit function instance to solve the system of equations
ifcn = rootfinder('ifcn', 'newton', vfcn_sx)
V = ifcn(MX(),X0,P)
X = [X0 if r==0 else V[(r-1)*nx:r*nx] for r in range(d+1)]

# Get an expression for the state at the end of the finie element
XF = 0
for r in range(d+1):
  XF += D[r]*X[r]

# Get the discrete time dynamics
F = Function('F', [X0,P],[XF])

# Do this iteratively for all finite elements
X = X0
for i in range(n):
  X = F(X,P)

# Fixed-step integrator
irk_integrator = Function('irk_integrator', {'x0':X0, 'p':P, 'xf':X},
                          integrator_in(), integrator_out())

# Create a convensional integrator for reference
ref_integrator = integrator('ref_integrator', 'cvodes', dae, 0, tf)

# Test values
x0_val  = N.array([0,1,0])
p_val = 0.2

# Make sure that both integrators give consistent results
for F in (irk_integrator,ref_integrator):
  print('-------')
  print('Testing ' + F.name())
  print('-------')

  # Generate a new function that calculates forward and reverse directional derivatives
  dF = F.factory('dF', ['x0', 'p', 'fwd:x0', 'fwd:p', 'adj:xf'],
                       ['xf', 'fwd:xf', 'adj:x0', 'adj:p']);
  arg = {}

  # Pass arguments
  arg['x0'] = x0_val
  arg['p'] = p_val

  # Forward sensitivity analysis, first direction: seed p and x0[0]
  arg['fwd_x0'] = [1,0,0]
  arg['fwd_p'] = 1

  # Adjoint sensitivity analysis, seed xf[2]
  arg['adj_xf'] = [0,0,1]

  # Integrate
  res = dF(**arg)

  # Get the nondifferentiated results
  print('%30s = %s' % ('xf', res['xf']))

  # Get the forward sensitivities
  print('%30s = %s' % ('d(xf)/d(p)+d(xf)/d(x0[0])', res['fwd_xf']))

  # Get the adjoint sensitivities
  print('%30s = %s' % ('d(xf[2])/d(x0)', res['adj_x0']))
  print('%30s = %s' % ('d(xf[2])/d(p)', res['adj_p']))



# This is an automatically generated file converting from the apm format

import casadi as cs

def hs055():
    # The optimal objective is (if given in):
    x_opt = cs.DM([0, 4/3, 5/3, 1, 2/3, 1/3])
    f_opt = 19/3
    x = cs.MX.sym('x', 6)
    obj = cs.MX.zeros(1, 1)
    x0 = cs.DM.zeros(6, 1)
    lbx = -cs.inf*cs.DM.ones(6, 1)
    ubx = cs.inf*cs.DM.ones(6, 1)
    u = cs.DM.zeros(6, 1)
    g = cs.MX.zeros(6, 1)
    lbg = -cs.inf*cs.DM.ones(6, 1)
    ubg = cs.inf*cs.DM.ones(6, 1)
    
    x0[0]= 1
    lbx[0]=0
    ubx[0]=1
    x0[1]= 2
    lbx[1]=0
    x0[2:5]= 0
    lbx[2:5]=0
    lbx[2] = 1 # revised not original bound
    ubx[3] =  1
    x0[5]= 2

    lbg[0] =  6
    ubg[0] =  6
    g[0] = x[0] + 2*x[1] + 5*x[4]
    lbg[1] =  3
    ubg[1] =  3
    g[1] = x[0] + x[1] + x[2]
    lbg[2] =  2
    ubg[2] =  2
    g[2] = x[3] + x[4] + x[5]
    lbg[3] =  1
    ubg[3] =  1
    g[3] = x[0] + x[3]
    lbg[4] =  2
    ubg[4] =  2
    g[4] = x[1] + x[4]
    lbg[5] =  2
    ubg[5] =  2
    g[5] = x[2] + x[5]
    f = x[0] + 2*x[1] + 4*x[4] + cs.exp(x[0]*x[3])
    
    return (x_opt, f_opt, x, f, g, lbg, ubg, lbx, ubx, x0)



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
# -*- coding: utf-8 -*-
from casadi import *

# Tests the finite difference implementation in CasADi
# Joel Andersson, UW Madison 2017

# Try with different scaling factors for x
a_test = [1e-10, 1e-4, 1., 1e4, 1e10]

for a in a_test:
    print()
    print("*** Testing a=%g ***" % a)

    # The following function is defined for the interval [0,1] and gives NaN outside it
    x = SX.sym('x');
    r = if_else(x<=1, if_else(x>=0, 2*x*a+cos(x*a), log(x)), log(1-x))/a

    # Test points in the domain, on the boundary and outside it
    x_test = [-1., 0., 0.4, 1., 1.4]
    print("%30s" % "x0", end='')
    for x0 in x_test: print("%15g " % x0, end='')
    print()

    # Try all supported schemes
    fd_methods = []
    fd_methods.append(('analytical', 0))
    fd_methods.append(('forward', 0))
    fd_methods.append(('backward', 0))
    fd_methods.append(('central', 0))
    fd_methods.append(('central', 1))
    fd_methods.append(('central', 2))
    fd_methods.append(('central', 5))
    fd_methods.append(('central', 20))
    fd_methods.append(('smoothing', 0))
    fd_methods.append(('smoothing', 1))
    fd_methods.append(('smoothing', 2))
    fd_methods.append(('smoothing', 5))
    fd_methods.append(('smoothing', 20))
    for (fd_method, h_iter) in fd_methods:
        descr = fd_method
        if h_iter!=0: descr += ' with ' + str(h_iter) + ' h updates'
        print("%30s" % descr, end='')
        # Construct function and differentiate
        opts = dict()
        if fd_method!='analytical':
            opts["enable_fd"]=True # enable finite differencing
            opts["enable_forward"]=False # disable forward mode AD
            opts["enable_reverse"]=False # disable reverse mode AD
            opts["enable_jacobian"]=False # disable AD by calculating full Jacobian
            opts["fd_method"]=fd_method # specify FD scheme
            if fd_method in ['central','smoothing']: opts["fd_options"] = dict(h_iter=h_iter)
        f = Function('f', [x], [r], ['x'], ['r'], opts)
        fwd_f = f.forward(1)
        # Evaluate
        for x0 in x_test:
            r0 = f(x0)
            d = fwd_f(x0, r0, 1.)
            print("%15g " % float(d), end='')
        print()



version_parts = {}

import sys
import os
import re

cd = os.getcwd()

if len(sys.argv)>1:
  cd = sys.argv[1]

with open(os.path.join(cd,"CMakeLists.txt"),"r") as f:
  for line in f.readlines():
    if "_VERSION" in line:
      m = re.search(r"set\(CASADI_(\w+)_VERSION (\d+)", line)
      if m:
        version_parts[m.group(1).lower()] = m.group(2)

sys.stdout.write("{major}.{minor}.{patch}".format(**version_parts))



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

#
# Exact Hessian
# =====================

from casadi import *
from numpy import *
import casadi as c

# We will investigate the use of an exact Hessian with the help of the Rosenbrock function

x=SX.sym('x')
y=SX.sym('y')
obj = (1-x)**2+100*(y-x**2)**2
constr = x**2+y**2
nlp={'x':vertcat(x,y), 'f':obj, 'g':constr}

# We solve the problem with an exact Hessian (default)

solver = nlpsol('solver', 'ipopt', nlp)
sol = solver(lbx=-10, ubx=10, lbg=0, ubg=1)
print('Optimal solution (exact Hessian): %s' % sol['x'])

# Same problem but with limited memory BFSG

solver = nlpsol('solver', 'ipopt', nlp, {'ipopt.hessian_approximation':'limited-memory'})
sol = solver(lbx=-10, ubx=10, lbg=0, ubg=1)
print('Optimal solution (BFGS): %s' % sol['x'])



#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
#
# -*- coding: utf-8 -*-
from casadi import *
#
# How to use Callback
# Joel Andersson
#

class MyCallback(Callback):
  def __init__(self, name, d, opts={}):
    Callback.__init__(self)
    self.d = d
    self.construct(name, opts)

  # Number of inputs and outputs
  def get_n_in(self): return 1
  def get_n_out(self): return 1

  # Initialize the object
  def init(self):
     print('initializing object')

  # Evaluate numerically
  def eval(self, arg):
    x = arg[0]
    f = sin(self.d*x)
    return [f]

# Use the function
f = MyCallback('f', 0.5)
res = f(2)
print(res)

# You may call the Callback symbolically
x = MX.sym("x")
print(f(x))

# Derivates OPTION 1: finite-differences
eps = 1e-5
print((f(2+eps)-f(2))/eps)

f = MyCallback('f', 0.5, {"enable_fd":True})
J = Function('J',[x],[jacobian(f(x),x)])
print(J(2))

# Derivates OPTION 2: Supply forward mode
# Example from https://www.youtube.com/watch?v=mYOkLkS5yqc&t=4s

class Example4To3(Callback):
  def __init__(self, name, opts={}):
    Callback.__init__(self)
    self.construct(name, opts)

  def get_n_in(self): return 1
  def get_n_out(self): return 1

  def get_sparsity_in(self,i):
    return Sparsity.dense(4,1)

  def get_sparsity_out(self,i):
    return Sparsity.dense(3,1)

  # Evaluate numerically
  def eval(self, arg):
    a,b,c,d = vertsplit(arg[0])
    ret = vertcat(sin(c)*d+d**2,2*a+c,b**2+5*c)
    return [ret]


class Example4To3_Fwd(Example4To3):
  def has_forward(self,nfwd):
    # This example is written to work with a single forward seed vector
    # For efficiency, you may allow more seeds at once
    return nfwd==1
  def get_forward(self,nfwd,name,inames,onames,opts):
    
    class ForwardFun(Callback):
      def __init__(self, opts={}):
        Callback.__init__(self)
        self.construct(name, opts)

      def get_n_in(self): return 3
      def get_n_out(self): return 1

      def get_sparsity_in(self,i):
        if i==0: # nominal input
          return Sparsity.dense(4,1)
        elif i==1: # nominal output
          return Sparsity(3,1)
        else: # Forward seed
          return Sparsity.dense(4,1)

      def get_sparsity_out(self,i):
        # Forward sensitivity
        return Sparsity.dense(3,1)

      # Evaluate numerically
      def eval(self, arg):
        a,b,c,d = vertsplit(arg[0])
        a_dot,b_dot,c_dot,d_dot = vertsplit(arg[2])
        print("Forward sweep with", a_dot,b_dot,c_dot,d_dot)
        w0 = sin(c)
        w0_dot = cos(c)*c_dot
        w1 = w0*d
        w1_dot = w0_dot*d+w0*d_dot
        w2 = d**2
        w2_dot = 2*d_dot*d
        r0 = w1+w2
        r0_dot = w1_dot + w2_dot
        w3 = 2*a
        w3_dot = 2*a_dot
        r1 = w3+c
        r1_dot = w3_dot+c_dot
        w4 = b**2
        w4_dot = 2*b_dot*b
        w5 = 5*w0
        w5_dot = 5*w0_dot
        r2 = w4+w5
        r2_dot = w4_dot + w5_dot
        ret = vertcat(r0_dot,r1_dot,r2_dot)
        return [ret]
    # You are required to keep a reference alive to the returned Callback object
    self.fwd_callback = ForwardFun()
    return self.fwd_callback

    
f = Example4To3_Fwd('f')
x = MX.sym("x",4)
J = Function('J',[x],[jacobian(f(x),x)])
print(J(vertcat(1,2,0,3)))

# Derivates OPTION 3: Supply reverse mode

class Example4To3_Rev(Example4To3):
  def has_reverse(self,nadj):
    # This example is written to work with a single forward seed vector
    # For efficiency, you may allow more seeds at once
    return nadj==1
  def get_reverse(self,nfwd,name,inames,onames,opts):
    
    class ReverseFun(Callback):
      def __init__(self, opts={}):
        Callback.__init__(self)
        self.construct(name, opts)

      def get_n_in(self): return 3
      def get_n_out(self): return 1

      def get_sparsity_in(self,i):
        if i==0: # nominal input
          return Sparsity.dense(4,1)
        elif i==1: # nominal output
          return Sparsity(3,1)
        else: # Reverse seed
          return Sparsity.dense(3,1)

      def get_sparsity_out(self,i):
        # Reverse sensitivity
        return Sparsity.dense(4,1)

      # Evaluate numerically
      def eval(self, arg):
        a,b,c,d = vertsplit(arg[0])
        r0_bar,r1_bar,r2_bar = vertsplit(arg[2])
        print("Reverse sweep with", r0_bar, r1_bar, r2_bar)
        w0 = sin(c)
        w1 = w0*d
        w2 = d**2
        r0 = w1+w2
        w3 = 2*a
        r1 = w3+c
        w4 = b**2
        w5 = 5*w0
        r2 = w4+w5
        w4_bar = r2_bar
        w5_bar = r2_bar
        w0_bar = 5*w5_bar
        b_bar = 2*b*w4_bar
        w3_bar = r1_bar
        c_bar = r1_bar
        a_bar = 2*w3_bar
        w1_bar = r0_bar
        w2_bar = r0_bar
        d_bar = 2*d*w2_bar
        w0_bar = w0_bar + w1_bar*d
        d_bar = d_bar + w0*w1_bar
        c_bar = c_bar + cos(c)*w0_bar
        ret = vertcat(a_bar,b_bar,c_bar,d_bar)
        return [ret]
    # You are required to keep a reference alive to the returned Callback object
    self.rev_callback = ReverseFun()
    return self.rev_callback

    
f = Example4To3_Rev('f')
x = MX.sym("x",4)
J = Function('J',[x],[jacobian(f(x),x)])
print(J(vertcat(1,2,0,3)))

# Derivates OPTION 4: Supply full Jacobian

class Example4To3_Jac(Example4To3):
  def has_jacobian(self): return True
  def get_jacobian(self,name,inames,onames,opts):
    class JacFun(Callback):
      def __init__(self, opts={}):
        Callback.__init__(self)
        self.construct(name, opts)

      def get_n_in(self): return 2
      def get_n_out(self): return 1

      def get_sparsity_in(self,i):
        if i==0: # nominal input
          return Sparsity.dense(4,1)
        elif i==1: # nominal output
          return Sparsity(3,1)

      def get_sparsity_out(self,i):
        return sparsify(DM([[0,0,1,1],[1,0,1,0],[0,1,1,0]])).sparsity()

      # Evaluate numerically
      def eval(self, arg):
        a,b,c,d = vertsplit(arg[0])
        ret = DM(3,4)
        ret[0,2] = d*cos(c)
        ret[0,3] = sin(c)+2*d
        ret[1,0] = 2
        ret[1,2] = 1
        ret[2,1] = 2*b
        ret[2,2] = 5
        return [ret]

    # You are required to keep a reference alive to the returned Callback object
    self.jac_callback = JacFun()
    return self.jac_callback

f = Example4To3_Jac('f')
x = MX.sym("x",4)
J = Function('J',[x],[jacobian(f(x),x)])
print(J(vertcat(1,2,0,3)))

    



# Python
from casadi import *

# Symbolic representation
x=SX.sym('x')
y=SX.sym('y')
z=y-(1-x)**2
f=x**2+100*z**2
P=dict(x=vertcat(x,y),f=f)

# Create solver instance
F=nlpsol('F','ipopt',P)

# Solve the problem
r=F(x0=[2.5,3.0])
print(r['x'])



# This is an automatically generated file converting from the apm format

import casadi as cs

def hs045():
    # The optimal objective is (if given in):
    x_opt = cs.DM([1,2,3,4,5])
    f_opt = 1
    x = cs.MX.sym('x', 5)
    obj = cs.MX.zeros(1, 1)
    x0 = cs.DM.zeros(5, 1)
    lbx = -cs.inf*cs.DM.ones(5, 1)
    ubx = cs.inf*cs.DM.ones(5, 1)
    g = []
    lbg = []
    ubg = []
    
    x0[0]= 0
    lbx[0]=0
    ubx[0]=1
    x0[1]= 0
    lbx[1]=0
    ubx[1]=2
    x0[2]= 0
    lbx[2]=0
    ubx[2]=3
    x0[3]= 0
    lbx[3]=0
    ubx[3]=4
    x0[4]= 0
    lbx[4]=0
    ubx[4]=5
    
    obj = 2 - x[0]*x[1]*x[2]*x[3]*x[4]/120
    
    
    f = obj
    
    return (x_opt, f_opt, x, f, g, lbg, ubg, lbx, ubx, x0)



#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
#
from casadi import *
import pylab as plt

# Height and velocity of ball
h = SX.sym('h')
v = SX.sym('v')
x = vertcat(h, v)

# ODE right-hand-side
hdot = v
vdot = -9.81
xdot = vertcat(hdot, vdot)

# Event indicator, trigger when it becomes negative
event_indicator = h

# DAE problem structure, with zero-crossing output
dae = dict(x = x, ode = xdot, zero = event_indicator)

# Event transition function
post_x = vertcat(h, -0.8*v)
transition = Function('transition', dict(x = x, post_x = post_x),
                      event_in(), event_out())

# Create an integrator instance for integrating over 7s
tgrid = np.linspace(0, 7, 100)
sim = integrator('sim', 'cvodes', dae, 0, tgrid,
                 dict(transition = transition))

# Simulate with initial height of 5
x0 = [5, 0]
simres = sim(x0 = x0)

# Visualize the solution
plt.figure(1)
plt.clf()
plt.plot(tgrid, simres['xf'][0, :].T)
plt.grid()
plt.show()



#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
#
# -*- coding: utf-8 -*-
import casadi as ca
from matplotlib import pyplot as plt
import numpy as np

# Simulating and optimizing a spring that breaks with DaeBuilder and event handling
# Joel Andersson, 2024-2025

# Start with an empty DaeBuilder instance
dae = ca.DaeBuilder('breaking_spring')

# Model variables
m = dae.add('m', 'parameter', 'tunable', dict(start = 1, description = 'Mass'))
v = dae.add('v', dict(start = -5, description = 'Velocity'))
x = dae.add('x', dict(start = -1, description = 'Displacement'))
k = dae.add('k', 'parameter', 'tunable', dict(start = 2, description = 'Spring constant'))
c = dae.add('c', 'parameter', 'tunable', dict(start = 0.1, description = 'Damping constant'))
d = dae.add('d', 'input', dict(start = 0, description = 'Disturbance'))
f = dae.add('f', dict(description = 'Spring force'))
b = dae.add('b', dict(type = 'Boolean', start = False, description = 'Is the spring broken?'))

# Equations
dae.eq(dae.der(x), v)
dae.eq(f, ca.if_else(ca.logic_not(b), -k*x + d, 0))
dae.eq(dae.der(v), (f - c*v) / m)
dae.when(x > 2, [dae.assign('b', True)])
dae.disp(True)

# Create an integrator in CasADi for hybrid simulation
tgrid = np.linspace(0, 4, 101)
simopts = dict(transition = dae.transition(), verbose = False, event_tol = 1e-12)
sim = ca.integrator('sim', 'cvodes', dae.create(), 0, tgrid, simopts)
sim

# Disturbance trajectory
d_traj = np.zeros(tgrid.shape)
d_traj[20:30] = -5
d_traj[40:50] = 15

# Simulate the system
res = sim(x0 = dae.start(dae.x()), p = dae.start(dae.p()), u = d_traj)

# Visualize the solution
fig, [ax_x, ax_u] = plt.subplots(2, 1, figsize = [10, 10])
ax_x.plot(tgrid, res['xf'].T, label = dae.x())
ax_x.legend()
ax_x.grid(True)
ax_x.set_title('States')
ax_u.step(tgrid, d_traj, label = dae.u()[0])
ax_u.legend()
ax_u.set_title('Disturbance')
ax_u.grid(True)
plt.show()



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#

import casadi as c
import numpy
from numpy import random, array, linalg, matrix, zeros, ones
import unittest
from types import *
from helpers import *
from casadi import *

scipy_available = True
try:
	from scipy.sparse import csr_matrix
except:
	scipy_available = False

class SXtests(casadiTestCase):

  def setUp(self):
    self.pool=FunctionPool()
    self.pool.append(lambda x: sqrt(x[0]),sqrt,"sqrt")
    self.pool.append(lambda x: sin(x[0]),sin,"sin")
    self.pool.append(lambda x: cos(x[0]),cos,"cos")
    self.pool.append(lambda x: tan(x[0]),tan,"tan")
    self.pool.append(lambda x: fabs(x[0]),fabs,"fabs")
    self.pool.append(lambda x: sign(x[0]),sign,"sign")
    self.pool.append(lambda x: arctan(x[0]),arctan,"arctan")
    self.pool.append(lambda x: arcsin(x[0]),arcsin,"arcsin")
    self.pool.append(lambda x: arccos(x[0]),arccos,"arccos")
    self.pool.append(lambda x: exp(x[0]),exp,"exp")
    self.pool.append(lambda x: log(x[0]),log,"log")
    self.pool.append(lambda x: x[0]**0,lambda x : x**0,"x^0",flags={'nozero'})
    self.pool.append(lambda x: x[0]**1,lambda x : x**1,"^1")
    self.pool.append(lambda x: x[0]**(-2),lambda x : x**(-2),"^-2",flags={'nozero'})
    self.pool.append(lambda x: x[0]**(0.3),lambda x : x**(0.3),"^0.3")
    self.pool.append(lambda x: floor(x[0]),floor,"floor")
    self.pool.append(lambda x: ceil(x[0]),ceil,"ceil")
    self.pool.append(lambda x: log1p(x[0]),np.log1p,"log1p")
    self.pool.append(lambda x: expm1(x[0]),np.expm1,"log1p")
    self.Jpool=FunctionPool()
    self.Jpool.append(lambda x: sqrt(x[0]),lambda x:diag(1/(2.0*sqrt(x))),"sqrt")
    self.Jpool.append(lambda x: sin(x[0]),lambda x:diag(cos(x)),"sin")
    self.Jpool.append(lambda x: fabs(x[0]),lambda x:diag(sign(x)),"fabs")
    self.Jpool.append(lambda x: sign(x[0]),lambda x:diag(x*0),"fabs")
    self.Jpool.append(lambda x: cos(x[0]),lambda x:diag(-sin(x)),"cos")
    self.Jpool.append(lambda x: tan(x[0]),lambda x:diag(1.0/cos(x)**2),"tan")
    self.Jpool.append(lambda x: arctan(x[0]),lambda x:diag( 1.0/(x**2+1)),"arctan")
    self.Jpool.append(lambda x: arcsin(x[0]),lambda x:diag( 1.0/sqrt(1-x**2)),"arcsin")
    self.Jpool.append(lambda x: arccos(x[0]),lambda x: diag(-1.0/sqrt(1-x**2)),"arccos")
    self.Jpool.append(lambda x: exp(x[0]),lambda x: diag(exp(x)),"exp")
    self.Jpool.append(lambda x: log(x[0]),lambda x: diag(1.0/x),"log")
    self.Jpool.append(lambda x: x[0]**0,lambda x :diag(zeros(x.shape)),"x^0")
    self.Jpool.append(lambda x: x[0]**1,lambda x : diag(ones(x.shape)),"^1")
    self.Jpool.append(lambda x: x[0]**(-2),lambda x : diag(-2.0/x**3),"^-2")
    self.Jpool.append(lambda x: x[0]**(0.3),lambda x :diag( 0.3/x**0.7),"^0.3")
    self.Jpool.append(lambda x: log1p(x[0]),lambda x :diag( 1.0/(1+x)),"log1p")
    self.Jpool.append(lambda x: expm1(x[0]),lambda x :diag( exp(x)),"log1p")
    self.matrixpool=FunctionPool()
    self.matrixpool.append(lambda x: norm_2(x[0]),linalg.norm,"norm_2")
    self.matrixbinarypool=FunctionPool()
    self.matrixbinarypool.append(lambda a: a[0]+a[1],lambda a: a[0]+a[1],"Matrix+Matrix")
    self.matrixbinarypool.append(lambda a: a[0]-a[1],lambda a: a[0]-a[1],"Matrix-Matrix")
    self.matrixbinarypool.append(lambda a: a[0]*a[1],lambda a: a[0]*a[1],"Matrix*Matrix")
    self.matrixbinarypool.append(lambda a: fmax(a[0],a[1]),lambda a: fmax(a[0],a[1]),"fmin")
    self.matrixbinarypool.append(lambda a: fmin(a[0],a[1]),lambda a: fmin(a[0],a[1]),"fmax")
    self.matrixbinarypool.append(lambda a: hypot(a[0],a[1]),lambda a: np.hypot(a[0],a[1]),"hypot")
    #self.matrixbinarypool.append(lambda a: dot(a[0],trans(a[1])),lambda a: dot(a[0].T,a[1]),name="dot(Matrix,Matrix)")
    self.matrixbinarypool.append(lambda a: mtimes(a[0],a[1].T),lambda a: np.dot(a[0],a[1].T),"dot(Matrix,Matrix.T)")

    #self.pool.append(lambda x: erf(x[0]),erf,"erf") # numpy has no erf

  def test_equivalence(self):
    x = SX.sym("x")
    y = SX.sym("y")    
    for expr,equiv in [(log1p(x),log(1+x)),(expm1(x),exp(x)-1),(hypot(x,y),sqrt(x**2+y**2))]:
      f=Function("f",[x,y],[expr])
      equiv_f = Function("equiv_f",[x,y],[equiv])
      self.checkfunction(f,equiv_f,inputs=[1.3,1.7])

  def test_scalarSX(self):
      x=SX.sym("x")
      x0=0.738

      self.numpyEvaluationCheckPool(self.pool,[x],x0,name="scalarSX")

  def test_gradient(self):
      self.message("jacobian of SX**number")
      x=SX.sym("x");
      x0=1;
      p=3 # increase to 20 to showcase ticket #56
      y=x**p;
      dx=jacobian(y,x);
      dxr=p;
      self.evaluationCheck([dx],dxr,[x],x0,name="jacobian");
      dxr=1
      for i in list(range(p)):
        y=jacobian(y,x)
        dxr=dxr*(p-i)


      self.evaluationCheck([y],dxr,[x],x0,name="recursive jacobian");

  def test_gradient2(self):
      self.message("jacobian of SX**SX")
      x=SX.sym("x");
      p=SX.sym("p");
      x0=1;
      p0=3 # increase to 20 to showcase ticket #56
      y=x**p;
      dx=jacobian(y,x);
      #print dx
      dxr=p0;
      self.evaluationCheck([dx],dxr,[x,p],[x0,p0],name="jacobian");

      dxr=1
      for i in list(range(p0)):
        y=jacobian(y,x)
        dxr=dxr*(p0-i)

      self.evaluationCheck([y],dxr,[x,p],[x0,p0],name="jacobian");

  def test_SXJacobian(self):
      self.message("SX(1,1) unary operation, jacobian")
      x=SX.sym("x")
      x0=array([[0.738]])

      def fmod(f,x):
        return jacobian_old(f, 0, 0)

      self.numpyEvaluationCheckPool(self.Jpool,[x],x0,name="SX unary operations, jacobian",fmod=fmod)

  def test_SXJac(self):
      self.message("SX(1,1) unary operation, jac")
      x=SX.sym("x")
      x0=array([[0.738]])

      def fmod(f,x):
          y = f.call(x)
          J = Function('J', x, [jacobian(y[0],x[0])])
          return J

      self.numpyEvaluationCheckPool(self.Jpool,[x],x0,name="SX unary operations, jac",fmod=fmod)

  def test_SXJacobians(self):
      self.message("SX(3,1) unary operation, jacobian")
      x=SX.sym("x",3)
      x0=array([0.738,0.9,0.3])

      def fmod(f,x):
        return jacobian_old(f, 0, 0)

      self.numpyEvaluationCheckPool(self.Jpool,[x],x0,name="SX unary operations, jacobian",fmod=fmod)

  def test_SXJacobians2(self):
      self.message("SX(1,3) unary operation, jacobian")
      x=SX.sym("x",1,3)

      x0=array([0.738,0.9,0.3])

      def fmod(f,x):
        return jacobian_old(f, 0, 0)

      self.numpyEvaluationCheckPool(self.Jpool,[x],x0,name="SX unary operations, jacobian",fmod=fmod)

  def test_SX(self):
      self.message("SX unary operations")
      x=SX.sym("x",3,2)
      x0=array([[0.738,0.2],[ 0.1,0.39 ],[0.99,0.999999]])

      self.numpyEvaluationCheckPool(self.pool,[x],x0,name="SX")

      x=SX.sym("x",3,3)
      x0=array([[0.738,0.2,0.3],[ 0.1,0.39,-6 ],[0.99,0.999999,-12]])
      #self.numpyEvaluationCheck(lambda x: c.det(x[0]), lambda   x: linalg.det(x),[x],x0,name="det(SX)")
      self.numpyEvaluationCheck(lambda x: SX(c.det(x[0])), lambda   x: linalg.det(x),[x],x0,name="det(SX)")
      self.numpyEvaluationCheck(lambda x: c.inv(x[0]), lambda   x: linalg.inv(x),[x],x0,name="inv(SX)")

  def test_SXSparse(self):
      self.message("SX unary operations, sparse")
      x=SX.sym("x")
      y=SX.sym("y")
      z=SX.sym("z")
      x=SX(Sparsity(4,3,[0,2,2,3],[1,2,1]),vertcat(*[x,y,z]))
      if scipy_available:
        x0=DM(Sparsity(4,3,[0,2,2,3],[1,2,1]),[0.738,0.1,0.99]).sparse()

        self.numpyEvaluationCheckPool(self.pool,[x],array(x0.todense()),name="SX",setx0=x0,excludeflags={'nozero'})
      else:
        x0=DM(Sparsity(4,3,[0,2,2,3],[1,2,1]),[0.738,0.1,0.99]).full()

        self.numpyEvaluationCheckPool(self.pool,[x],x0,name="SX",setx0=x0,excludeflags={'nozero'})

  def test_SXbinary(self):
      self.message("SX binary operations")
      x=SX.sym("x",3,2)
      y=SX.sym("x",3,2)
      x0=array([[0.738,0.2],[ 0.1,0.39 ],[0.99,0.999999]])
      y0=array([[1.738,0.6],[ 0.7,12 ],[0,-6]])
      self.numpyEvaluationCheckPool(self.matrixbinarypool,[x,y],[x0,y0],name="SX")
      self.assertRaises(RuntimeError, lambda : mtimes(x,y))

  def test_SXbinary_codegen(self):
      self.message("SX binary operations")
      x=SX.sym("x",4,2)
      y=SX.sym("x",4,2)
      x0=array([[0.738,0.2],[ 0.1,0.39 ],[0.99,0.999999],[1,2]])
      y0=array([[1.738,0.6],[ 0.7,12 ],[0,-6],[1,2]])
      for f in self.matrixbinarypool.casadioperators:
        f = Function('f',[x,y],[f([x,y])])
        self.check_codegen(f,inputs=[x0,y0],std="c99")
        self.check_codegen(f,inputs=[x0,y0],std="c89")

  def test_SXbinary_diff(self):
      self.message("SX binary operations")
      x=SX.sym("x",4,2)
      y=SX.sym("x",4,2)
      dx=SX.sym("x",4,2)
      dy=SX.sym("x",4,2)
      x0=array([[0.738,0.2],[ 0.1,0.39 ],[0.99,0.999999],[1,2]])
      y0=array([[1.738,0.6],[ 0.7,12 ],[0,-6],[1,2]])
      for f,name in zip(self.matrixbinarypool.casadioperators,self.matrixbinarypool.names):
        f = Function('f',[x,y],[f([x,y])])
        f.disp(True)
        df = Function('f',[x,y,dx,dy],[jtimes(f(x,y),x,dx)+jtimes(f(x,y),y,dy)])
        eps = 1e-7
        DM.rng(0)
        dx0 = DM.rand(x.sparsity())
        dy0 = dx0 if name in ["fmin","fmax"] else DM.rand(y.sparsity())
        dy0 = dx0 if name in ["fmin","fmax"] else DM.rand(y.sparsity())
        df_experiment = (f(x0+eps*dx0,y0+eps*dy0)-f(x0,y0))/eps
        print(df_experiment,df(x0,y0,dx0,dy0))
        self.checkarray(df(x0,y0,dx0,dy0),df_experiment,digits=4)

  def test_DMbinary(self):
      self.message("SX binary operations")
      x=SX.sym("x",3,2)
      y=SX.sym("x",3,2)
      x0=array([[0.738,0.2],[ 0.1,0.39 ],[0.99,0.999999]])
      y0=array([[1.738,0.6],[ 0.7,12 ],[0,-6]])
      for f,fr,label,flags in self.matrixbinarypool.zip():
        self.checkarray(f(vertcat(*[x0,y0])),fr(vertcat(*[x0,y0])),label)

  def test_SXbinarySparse(self):
      self.message("SX binary operations")
      x=SX.sym("x")
      y=SX.sym("y")
      z=SX.sym("z")
      x2=SX.sym("x2")
      y2=SX.sym("y2")
      z2=SX.sym("z2")
      xx=SX(Sparsity(4,3,[0,2,2,3],[1,2,1]),vertcat(*[x,y,z]))
      yy=SX(Sparsity(4,3,[0,2,2,3],[0,2,3]),vertcat(*[x2,z2,y2]))

      if scipy_available:
        x0=DM(Sparsity(4,3,[0,2,2,3],[1,2,1]),[0.738,0.1,0.99]).sparse()
        y0=DM(Sparsity(4,3,[0,2,2,3],[0,2,3]),[1.738,0.7,-6]).sparse()

        self.numpyEvaluationCheckPool(self.matrixbinarypool,[xx,yy],[array(x0.todense()),array(y0.todense())],name="SX",setx0=[x0,y0])
      else:
        x0=DM(Sparsity(4,3,[0,2,2,3],[1,2,1]),[0.738,0.1,0.99]).full()
        y0=DM(Sparsity(4,3,[0,2,2,3],[0,2,3]),[1.738,0.7,-6]).full()

        self.numpyEvaluationCheckPool(self.matrixbinarypool,[xx,yy],[x0,y0],name="SX",setx0=[x0,y0])
      self.assertRaises(RuntimeError, lambda : mtimes(xx,yy))


  @known_bug()  # Test refactoring, cf. #1436
  def test_SXslicing(self):
      self.message("SX slicing/indexing")
      x=SX.sym("x",3,2)
      x0=array([[0.738,0.2],[ 0.1,0.39 ],[0.99,0.999999]])

      self.message(":dense")
      self.numpyEvaluationCheck(lambda x: SX(x[0][0,0]), lambda x: matrix(x)[0,0],[x],x0,name="x[0,0]")
      self.numpyEvaluationCheck(lambda x: SX(x[0][1,0]), lambda x: matrix(x)[1,0],[x],x0,name="x[1,0]")
      self.numpyEvaluationCheck(lambda x: SX(x[0][0,1]), lambda x: matrix(x)[0,1],[x],x0,name="x[1,0]")
      self.numpyEvaluationCheck(lambda x: SX(x[0][0,-1]), lambda x: matrix(x)[0,-1],[x],x0,name="x[0,-1]")
      self.numpyEvaluationCheck(lambda x: x[0][:,0], lambda x: matrix(x)[:,0],[x],x0,name="x[:,0]")
      self.numpyEvaluationCheck(lambda x: x[0][:,1], lambda x: matrix(x)[:,1],[x],x0,name="x[:,1]")
      self.numpyEvaluationCheck(lambda x: x[0][1,:], lambda x: matrix(x)[1,:],[x],x0,name="x[1,:]")
      self.numpyEvaluationCheck(lambda x: x[0][0,:], lambda x: matrix(x)[0,:],[x],x0,name="x[0,:]")
      self.numpyEvaluationCheck(lambda x: x[0][-1,:], lambda x: matrix(x)[-1,:],[x],x0,name="x[-1,:]")
      self.numpyEvaluationCheck(lambda x: x[0][:,-2], lambda x: matrix(x)[:,-2],[x],x0,name="x[:,-2]")
      self.numpyEvaluationCheck(lambda x: x[0][0:-2,0:-1], lambda x: matrix(x)[0:-2,0:-1],[x],x0,name="x[0:-2,0:-1]")
      self.numpyEvaluationCheck(lambda x: x[0][0:2,0:2], lambda x: matrix(x)[0:2,0:2],[x],x0,name="x[0:2,0:2]")
      self.numpyEvaluationCheck(lambda x: x[0][[0,1],0:2], lambda x: matrix(x)[[0,1],0:2],[x],x0,name="x[[0,1],0:2]")
      self.numpyEvaluationCheck(lambda x: x[0].nz[[0,2,3]], lambda x: matrix([x[0,0],x[2,0],x[0,1]]).T,[x],x0,name="x[[0,2,3]]")

      myarray=array([0,2,3])
      mylist=list(myarray)
      #self.numpyEvaluationCheck(lambda x: x[0][mylist], lambda x: matrix([x[0,0],x[1,0],x[1,1]]).T,[x],x0,name="x[[0,2,3]]")
      self.numpyEvaluationCheck(lambda x: x[0].nz[0:2], lambda x: matrix(x.T.ravel()[0:2]).T,[x],x0,name="x[0:2] on dense matrix")
      self.numpyEvaluationCheck(lambda x: x[0].nz[1], lambda x: matrix(x.T.ravel()[1]).T,[x],x0,name="x[1]")
      self.numpyEvaluationCheck(lambda x: x[0].nz[-1], lambda x: matrix(x.ravel()[-1]).T,[x],x0,name="x[-1]")

      self.message(":sparse")

      x=SX(Sparsity(4,3,[0,2,2,3],[1,2,1]),vertcat(*[SX.sym("x"),SX.sym("y"),SX.sym("z")]))
      sx0=[0.738,0.39,0.99]
      x0=DM(Sparsity(4,3,[0,2,2,3],[1,2,1]),[0.738,0.39,0.99]).full()
      self.numpyEvaluationCheck(lambda x: SX(x[0][0,0]), lambda x: matrix(x)[0,0],[x],x0,name="x[0,0]",setx0=[sx0])
      self.numpyEvaluationCheck(lambda x: SX(x[0][0,0]), lambda x: matrix(x)[0,0],[x],x0,name="x[0,0]",setx0=[sx0])
      self.numpyEvaluationCheck(lambda x: SX(x[0][1,0]), lambda x: matrix(x)[1,0],[x],x0,name="x[1,0]",setx0=[sx0])
      self.numpyEvaluationCheck(lambda x: SX(x[0][0,1]), lambda x: matrix(x)[0,1],[x],x0,name="x[1,0]",setx0=[sx0])
      self.numpyEvaluationCheck(lambda x: SX(x[0][0,-1]), lambda x: matrix(x)[0,-1],[x],x0,name="x[0,-1]",setx0=[sx0])
      self.numpyEvaluationCheck(lambda x: x[0][:,0], lambda x: matrix(x)[:,0],[x],x0,name="x[:,0]",setx0=[sx0])
      self.numpyEvaluationCheck(lambda x: x[0][:,1], lambda x: matrix(x)[:,1],[x],x0,name="x[:,1]",setx0=[sx0])
      self.numpyEvaluationCheck(lambda x: x[0][1,:], lambda x: matrix(x)[1,:],[x],x0,name="x[1,:]",setx0=[sx0])
      self.numpyEvaluationCheck(lambda x: x[0][0,:], lambda x: matrix(x)[0,:],[x],x0,name="x[0,:]",setx0=[sx0])
      self.numpyEvaluationCheck(lambda x: x[0][-1,:], lambda x: matrix(x)[-1,:],[x],x0,name="x[-1,:]",setx0=[sx0])
      self.numpyEvaluationCheck(lambda x: x[0][:,-2], lambda x: matrix(x)[:,-2],[x],x0,name="x[:,-2]",setx0=[sx0])
      self.numpyEvaluationCheck(lambda x: x[0][0:-2,0:-1], lambda x: matrix(x)[0:-2,0:-1],[x],x0,name="x[0:-2,0:-1]",setx0=[sx0])
      self.numpyEvaluationCheck(lambda x: x[0][0:2,0:2], lambda x: matrix(x)[0:2,0:2],[x],x0,name="x[0:2,0:2]",setx0=[sx0])
      self.numpyEvaluationCheck(lambda x: x[0][[0,1],0:2], lambda x: matrix(x)[[0,1],0:2],[x],x0,name="x[[0,1],0:2]",setx0=[sx0])
      self.numpyEvaluationCheck(lambda x: x[0].nz[[2,1]], lambda x: matrix([x[1,2],x[2,0]]).T,[x],x0,name="x[[2,1]]")
      self.numpyEvaluationCheck(lambda x: x[0].nz[0:2], lambda x: matrix(sx0[0:2]).T,[x],x0,name="x[0:2] on dense matrix")
      self.numpyEvaluationCheck(lambda x: x[0].nz[1], lambda x: matrix(sx0[1]).T,[x],x0,name="x[1]",setx0=[sx0])
      self.numpyEvaluationCheck(lambda x: x[0].nz[-1], lambda x: matrix(sx0[-1]).T,[x],x0,name="x[-1]",setx0=[sx0])


  def test_SX1(self):
    self.message("SXFunction evaluation")
    fun=lambda x,y: [x+y,x*y,x**2+y**3]
    x=SX.sym("x")
    y=SX.sym("y")
    f=Function("f", [vertcat(*[x,y])],[vertcat(*fun(x,y))])
    L=[2,3]
    f_in = [0]*f.n_in();f_in[0]=L
    f_out = f.call(f_in)
    z=f_out[0].full()
    zr=fun(*L)
    for i in range(3):
      self.assertAlmostEqual(z[i], zr[i],10,'SXfunction output in correct')
    self.message("SXFunction jacobian evaluation")
    J = jacobian_old(f, 0, 0)
    J_in = [0]*J.n_in();J_in[0]=L
    J_out = J.call(J_in)
    Jr=np.array([[1,1],[3,2],[4,27]])
    self.checkarray(J_out[0],Jr,"SXfunction jacobian evaluates incorrectly")

  def test_SX2(self):
    self.message("SXFunction evalution 2")
    fun = lambda x,y: [3-sin(x*x)-y, sqrt(y)*x]
    # variables
    x = SX.sym("x")
    y = SX.sym("y")

    # Create function
    f = fun(x,y)
    if GlobalOptions.getSimplificationOnTheFly():
      self.assertEqual(str(f),'[SX(((3-sin(sq(x)))-y)), SX((sqrt(y)*x))]','SX representation is wrong '+str(f))
    else:
      self.assertEqual(str(f),'[SX(((3-sin((x*x)))-y)), SX((sqrt(y)*x))]','SX representation is wrong '+str(f))
    fcn = Function("fcn", [vertcat(*[x,y])],[vertcat(*f)])

    self.assertEqual(repr(fcn),'Function(fcn:(i0[2])->(o0[2]) SXFunction)','SX representation is wrong')

    # Pass inputs
    L=[2,3]
    fcn_in = [0]*fcn.n_in();fcn_in[0]=L

    # Evaluate numerically
    fcn_out = fcn.call(fcn_in)

    # Get the results
    res = tuple(fcn_out[0].nonzeros())
    self.assertAlmostEqual(res[0], fun(*L)[0],10,'SXfunction evaluation wrong')
    self.assertAlmostEqual(res[1], fun(*L)[1],10,'SXfunction evaluation wrong')

  def test_SX_func(self):
    self.message("Function constructors")
    x0=SX.sym("x")
    x1=SX.sym("x")
    x2=SX.sym("x")
    x3=SX.sym("x")
    x4=SX.sym("x")
    x5=SX.sym("x")
    x6=SX.sym("x")
    y=SX.sym("y",2,3)

    f=Function("f", [y],[y])
    self.checkarray(f.size_in(0),(2,3),"Function constructors")
    self.checkarray(f.size_out(0),(2,3),"Function constructors")

    self.assertRaises(TypeError if systemswig else NotImplementedError,lambda: Function("f", y,[y,y]))
    self.assertRaises(TypeError if systemswig else NotImplementedError,lambda: Function("f", x0,[x0,x1]))

  def test_evalfail(self):
    self.message("eval fail test")
    x = SX.sym("x",2,2)
    f = Function("f", [x], [x])
    self.assertRaises(TypeError if systemswig else NotImplementedError,lambda: f.call(x))

  def test_SXconversion(self):
    self.message("Conversions from and to SX")
    y=SX.sym("y")
    x=SX.sym("x",3,3)
    SX(y)
    SX(x)
    c.det(x)
    y=array(DM(x))
    c.det(y)

  def test_SXbool(self):
    self.message("bool")

    x = SX.sym("x")
    y = SX.sym("y")

    f = Function("f", [vertcat(*[x,y])],[vertcat(*[logic_and(x,y),logic_or(x,y),logic_not(x)])])


    for t1 in [0,1]:
      for t2 in [0,1]:
        T1 = t1!=0
        T2 = t2!=0
        f_in = [0]*f.n_in();f_in[0]=[t1,t2]
        f_out = f.call(f_in)
        self.checkarray(f_out[0],DM([T1 and T2,T1 or T2,not T1]),"bool(%d,%d): %s" % (t1,t2,str(f_out[0])))

  def test_SXineq(self):
    self.message("SX ineq")

    x = SX.sym("x")
    y = SX.sym("y")

    f = Function("f", [vertcat(*[x,y])],[vertcat(*[x<y,x<=y,x>=y,x==y,x!=y])])


    for t1 in [-10,0.1,0,1,10]:
      for t2 in [-10,0.1,0,1,10]:
        T1 = t1
        T2 = t2
        f_in = [0]*f.n_in();f_in[0]=[t1,t2]
        f_out = f.call(f_in)
        self.checkarray(f_out[0],DM([T1 < T2,T1 <= T2, T1 >= T2, T1 == T2, T1 != T2]),"ineq(%d,%d)" % (t1,t2))



  def test_SX_func2(self):
    self.message("SXmatrix typemaps constructors")
    simplify(SX.sym("x"))
    list = [    ("number",2.3, (1,1)),
                ("SX", SX.sym("x"), (1,1))
    ];
    for name, arg,shape in list:
      self.message(":" + name)
      i=c.transpose(c.transpose(arg))
      self.assertEqual(i.shape[0],shape[0],"shape mismatch")
      self.assertEqual(i.shape[1],shape[1],"shape mismatch")
      SX(arg).is_empty()

  def test_SX_func3(self):
    self.message("vector(SXmatrix) typemaps constructors")
    y=SX.sym("y")
    x=SX.sym("x",3,1)
    vertcat(*[x,x])
    vertcat(*[y,y])
    vertcat(*[x,[]])

  def test_eval(self):
    self.message("Function eval")
    x=SX.sym("x",2,2)
    y=SX.sym("y",2,2)
    f  = Function("f", [x,y], [x*y])
    f(x,y)

  def test_symbolcheck(self):
    self.message("Check if non-symbolic inputs are caught")
    self.assertRaises(RuntimeError, lambda : Function("f", [SX(0)],[SX.sym("x")]))

  def test_sparseconstr(self):
    self.message("Check sparsity constructors")
    self.checkarray(DM.ones(Sparsity.lower(3)).full(),np.array([[1,0,0],[1,1,0],[1,1,1]]),"tril")
    self.checkarray(DM.ones(Sparsity.diag(3)).full(),np.array([[1,0,0],[0,1,0],[0,0,1]]),"diag")

  def test_subsassignment(self):
    self.message("Check subscripted assignment")

    import numpy
    numpy.random.seed(42)
    xn = numpy.random.random((3,4))

    x=DM(xn)

    y=DM(7,8)
    z = numpy.zeros((7,8))
    y[0,0]=12; z[0,0] = 12
    self.checkarray(y,z,"scalar assignment")
    z[1:4,[2,4,5,6]]=xn
    y[1:4,[2,4,5,6]]=x
    self.checkarray(y,z,"range assignment")

    kl=[2,4,5,8]
    y.nz[kl]=1.0
    s=y.sparsity()
    for k in kl:
      z[s.row()[k],s.get_col()[k]]=1.0
    self.checkarray(y,z,"nonzero scalar assignment")
    y.nz[kl]=DM(kl)

    cnt=0
    for k in kl:
      z[s.row()[k],s.get_col()[k]]=kl[cnt]
      cnt+=1
    self.checkarray(y,z,"nonzero range assignment")

  @skip(not GlobalOptions.getSimplificationOnTheFly())
  def test_substitute(self):
    self.message("Basic symbolic algebra: substitute")
    x=SX.sym("x")
    y=SX.sym("y")
    z = cos(x)*y
    self.assertTrue(depends_on(z,y))
    self.assertTrue(depends_on(z,x))
    w = substitute(z,x,0)
    self.assertTrue(w.is_symbolic())
    self.assertTrue(depends_on(w,y))
    self.assertFalse(depends_on(w,x))
    self.assertTrue(is_equal(w,y))
    r=w-y
    self.assertFalse(r.is_symbolic())
    self.assertTrue(r.is_zero())
    self.assertEqual(float(r),0)
    self.assertEqual(float(r),0)
    y = SX.sym("y",2)
    y = substitute(y+6,y,0)
    self.assertEqual(int(y[0]),6)
    self.assertEqual(int(y[1]),6)

  def test_primitivefunctions(self):
    self.message("Primitive functions")
    x=SX.sym("x")

    nums = [-2,-1.5,-1,-0.5,-0.25,0,0.25,0.5,1,1.5,2]

    def test(fun,comment,nums,reference):
      self.message(":"+comment)
      f = Function("f", [x],[fun(x)])
      for n,r in zip(nums,reference):
        f_in = [0]*f.n_in();f_in[0]=n
        f_out = f.call(f_in)
        self.assertEqual(f_out[0][0],r)

    test(casadi.sign,"sign",nums,[-1,-1,-1,-1,-1,0,1,1,1,1,1])
    test(casadi.heaviside,"heaviside",nums,[0,0,0,0,0,0.5,1,1,1,1,1])
    test(casadi.ramp,"ramp",nums,[0,0,0,0,0,0,0.25,0.50,1,1.5,2])
    test(casadi.rectangle,"rectangle",nums,[0,0,0,0.5,1,1,1,0.5,0,0,0])
    test(casadi.triangle,"triangle",nums,[0,0,0,0.5,0.75,1,0.75,0.5,0,0,0])


  def test_taylor(self):
    self.message("univariate taylor expansion")
    x=SX.sym("x")

    if GlobalOptions.getSimplificationOnTheFly():
      self.assertTrue(is_equal(taylor(sin(x),x),x))

    a_=0.13
    x_=0.15

    a = SX.sym("a")

    def test(e,r):
      f = Function("f", [x,a],[e])
      f_in = [0]*f.n_in();f_in[0]=x_
      f_in[1]=a_
      f_out = f.call(f_in)
      self.assertAlmostEqual(f_out[0][0],r,10)

    test(taylor(sin(x),x,a,0),sin(a_))
    test(taylor(sin(x),x,a,1),sin(a_)+cos(a_)*(x_-a_))
    test(taylor(sin(x),x,a,2),sin(a_)+cos(a_)*(x_-a_)-(sin(a_)*(x_-a_)**2)/2.0)
    test(taylor(sin(x),x,a,3),sin(a_)+cos(a_)*(x_-a_)-(sin(a_)*(x_-a_)**2)/2.0-(cos(a_)*(x_-a_)**3)/6.0)

    M=blockcat([[a*sin(x),a*cos(x)],[exp(a*x),a*x**2],[cos(x),0]])
    f = Function("f", [x,a],[taylor(M,x)])
    f_in = [0]*f.n_in();f_in[0]=x_
    f_in[1]=a_
    f_out = f.call(f_in)
    self.checkarray(f_out[0],np.array([[x_*a_,a_],[1+a_*x_,0],[1,0]]),"taylor on dense matrices")

  def test_null(self):
    self.message("Function null")
    x = SX.sym("x")

    f = Function("f", [x],[x**2,[]])
    f_out = f.call([0])
    self.assertTrue(f_out[1].is_empty())

    f = Function("f", [x,[]],[x**2,[]])
    f_out = f.call([0,0])
    self.assertTrue(f_out[1].is_empty())
    f_out = f.call([0,0])

    r = f.call([x,[]])
    self.assertTrue(r[1].is_empty())

    r = f.call([x,[]])
    self.assertTrue(r[1].is_empty())

    r = f.call([x,SX(0,1)])
    self.assertTrue(r[1].is_empty())

    r = f.call([x,SX(1,0)])
    self.assertTrue(r[1].is_empty())

    #self.assertRaises(Exception,lambda : f([x,x]))
    #self.assertRaises(Exception,lambda : f([[],[]]))

  def test_mtaylor(self):
    self.message("multivariate taylor expansions")
    x=SX.sym("x")
    y=SX.sym("y")
    a=SX.sym("a")
    b=SX.sym("b")

    a_=0.13
    x_=0.15

    b_=0.73
    y_=0.75

    def test(e,r):
      f = Function("f", [vertcat(*[x,y]),vertcat(*[a,b])],[e])
      f_in = [0]*f.n_in();f_in[0]=[x_,y_]
      f_in[1]=[a_,b_]
      f_out = f.call(f_in)
      self.assertAlmostEqual(f_out[0][0],r,10)

    test(mtaylor(sin(x+y),vertcat(*[x,y]),vertcat(*[a,b]),0),sin(a_+b_))
    test(mtaylor(sin(x+y),vertcat(*[x,y]),vertcat(*[a,b]),1),sin(a_+b_)+(cos(b_+a_)*(x_-a_)+cos(b_+a_)*(y_-b_)))

    def sol(x,y,a,b):
      return sin(b+a)+(cos(b+a)*(x-a)+cos(b+a)*(y-b))-(sin(b+a)*(x-a)**2+2*sin(b+a)*(y-b)*(x-a)+sin(b+a)*(y-b)**2)/2

    test(mtaylor(sin(x+y),vertcat(*[x,y]),vertcat(*[a,b]),2),sol(x_,y_,a_,b_))

    def sol(x,y,a,b):
      return sin(b+a)+(cos(b+a)*(x-a)+cos(b+a)*(y-b))-(sin(b+a)*(x-a)**2+2*sin(b+a)*(y-b)*(x-a)+sin(b+a)*(y-b)**2)/2-(cos(b+a)*(x-a)**3+3*cos(b+a)*(y-b)*(x-a)**2+3*cos(b+a)*(y-b)**2*(x-a)+cos(b+a)*(y-b)**3)/6

    test(mtaylor(sin(x+y),vertcat(*[x,y]),vertcat(*[a,b]),3),sol(x_,y_,a_,b_))

    def sol(x,y,a,b):
      return (-2*sin(b+a)*(x-a)*(y-b)-sin(b+a)*(x-a)**2)/2+cos(b+a)*(y-b)-(cos(b+a)*(x-a)**3)/6+cos(b+a)*(x-a)+sin(b+a)

    test(mtaylor(sin(x+y),vertcat(*[x,y]),vertcat(*[a,b]),3,[1,2]),sol(x_,y_,a_,b_))

    test(mtaylor(sin(x+y),vertcat(*[x,y]),vertcat(*[0,0]),4,[1,2]),(-3*x_**2*y_-x_**3)/6+y_+x_)

  def test_issue107(self):
    self.message("Regression test for issue 107: +=")
    x=SX.sym("x")
    y=SX.sym("y")

    z=x
    z+=y

    self.assertTrue(x.is_symbolic())
    self.assertFalse(z.is_symbolic())

    x=SX.sym("x")
    y=SX.sym("y")

    z=x
    z+=y

    self.assertTrue(x.is_symbolic())
    self.assertFalse(z.is_symbolic())

  def test_evalchecking(self):
    x = SX.sym("x",1,5)

    y = SX.sym("y",1,3)
    z = SX.sym("z",5,1)
    q = SX.sym("z",1,6)

    f = Function("f", [x],[x**2])

    self.assertRaises(RuntimeError, lambda : f(y))
    self.assertRaises(RuntimeError, lambda : f(q))
    f(z)

  def test_indexinglimits(self):
    self.message("Limits of indexing")
    y = casadi.SX.sym("y", 3)
    self.assertRaises(RuntimeError,lambda : y[[0, 5]] )
    try:
      y[[0, 5]] = SX.sym("a")
      self.assertTrue(False)
    except RuntimeError:
      pass
    y[[0, 2]]
    y[[0, 2]] = SX.sym("a")

  def test_issue181(self):
    self.message("Regression test #181")
    x = SX.sym("x")
    #self.assertRaises(TypeError,lambda : SX([x,None]))  # FIXME: this is leaking memory
    self.assertRaises(TypeError if systemswig else NotImplementedError,lambda: Function("f", [[x], [None]], [[2 * x]]))

  @known_bug()  # Not implemented
  def test_is_equal(self):
    self.message("equivalent")
    x = SX.sym("x")
    a = x*x
    b = x*x
    self.assertTrue(a.is_equal(b,1))

  @skip(not GlobalOptions.getSimplificationOnTheFly())
  def test_SXsimplifications(self):
    self.message("simplifications")
    x = SX.sym("x")

    ops = []
    def temp(x):
      y = 0.5*x
      return y+y

    ops.append(temp)

    def temp(x):
      y = x/2
      return y+y

    ops.append(temp)

    def temp(x):
      y = x*0.5
      return y+y

    ops.append(temp)


    def temp(x):
      y = x*x
      return ((-y)/y)*(-x)

    ops.append(temp)

    ops.append(lambda x: ((-(x*x))/(x*x))*(-x))

    #ops.append(lambda x: ((-x*x)/(x*x))*(-x))

    def temp(x):
      y = x*x
      return (y/(-y))*(-x)

    ops.append(temp)

    def temp(x):
      y = x*x
      return ((-y)/(-y))*(x)

    ops.append(temp)
    ops.append(lambda x: (x-x) + x)
    ops.append(lambda x: ((x*x)-(x*x)) + x)
    ops.append(lambda x: 4*(0.25*x))
    ops.append(lambda x: 4*(x*0.25))
    ops.append(lambda x: 4*(0.25*x))
    ops.append(lambda x: 4*(x*0.25))
    ops.append(lambda x: (0.25*x)*4)
    ops.append(lambda x: (x*0.25)*4)
    ops.append(lambda x: (4*x)/4)
    ops.append(lambda x: 4*(x/4))
    ops.append(lambda x: (x/4)/0.25)
    ops.append(lambda x: x*(((4/x)*x)/4))
    ops.append(lambda x: x*((x*(2/x))/2))
    ops.append(lambda x: x*(((2*x)/x)/2))
    ops.append(lambda x: x*((x/(2*x))*2))
    ops.append(lambda x: x+0)
    ops.append(lambda x: 0+x)
    ops.append(lambda x: x-0)
    ops.append(lambda x: 0-(-x))
    ops.append(lambda x: x*1)
    ops.append(lambda x: 1*x)
    ops.append(lambda x: 1*(x*1))
    ops.append(lambda x: (1*x)*1)
    ops.append(lambda x: (0.5*x)+(0.5*x))
    ops.append(lambda x: (x/2)+(x/2))
    ops.append(lambda x: (x*0.5)+(0.5*x))
    ops.append(lambda x: (SX(4)-SX(4))+x)


    y = SX.sym("x")

    ops.append(lambda x: ((x+y)-(y+x))+x)
    ops.append(lambda x: ((x*y)-(y*x))+x)
    ops.append(lambda x: ((-x)-(-x))+x)

    for op in ops:
      y = op(x)
      f = Function("f", [x],[y])
      f_in = [0]*f.n_in();f_in[0]=0.3
      f_out = f.call(f_in)
      self.checkarray(f_out[0],array(DM(op(0.3))),"simplifications")
      self.assertEqual(str(y),"x")

      y = op(-x)
      f = Function("f", [x],[y])
      f_in = [0]*f.n_in();f_in[0]=0.3
      f_out = f.call(f_in)
      self.checkarray(f_out[0],array(DM(op(-0.3))),"simplifications")
      self.assertEqual(str(y),"(-x)")

  def test_truth(self):
    self.message("Truth values")
    self.assertRaises(Exception, lambda : bool(SX.sym("x")))
    self.assertRaises(Exception, lambda : bool(SX.sym("x")>0))
    self.assertTrue(bool(SX(1)))
    self.assertFalse(bool(SX(0)))
    self.assertTrue(bool(SX(0.2)))
    self.assertTrue(bool(SX(-0.2)))
    self.assertRaises(Exception, lambda : bool(SX.sym("x")))
    self.assertRaises(Exception, lambda : bool(SX.sym("x")>0))
    self.assertTrue(bool(SX(SX(1))))
    self.assertFalse(bool(SX(SX(0))))
    self.assertTrue(bool(SX(SX(0.2))))
    self.assertTrue(bool(SX(SX(-0.2))))
    self.assertRaises(Exception, lambda : bool(SX([2.0,3])))

  def test_if_else(self):
    x = SX.sym("x")
    y = if_else(x,1,2)
    f = Function("f", [x],[y])
    f_in = [0]*f.n_in();f_in[0]=1
    f_out = f.call(f_in)
    self.assertTrue(f_out[0]==1,"if_else")
    f_in = [0]*f.n_in();f_in[0]=0
    f_out = f.call(f_in)
    self.assertTrue(f_out[0]==2,"if_else")

    x0 = 2.1
    y = if_else(x>1,x**2,x**3)
    f = Function("f", [x],[y])
    f_in = [0]*f.n_in();f_in[0]=x0
    f_out = f.call(f_in)
    self.checkarray(f_out[0],x0**2,"if_else sens")

    x0 = -2.1
    f_in = [0]*f.n_in();f_in[0]=x0
    f_out = f.call(f_in)
    self.checkarray(f_out[0],x0**3,"if_else sens")


  def test_is_regular(self):
    x = SX.sym("x")

    self.assertTrue(SX(0).is_regular())
    self.assertFalse(SX(inf).is_regular())
    with self.assertRaises(Exception):
      self.assertTrue(x.nz[0])

    self.assertTrue(SX(DM([0,1])).is_regular())
    self.assertFalse(SX(DM([0,inf])).is_regular())
    self.assertFalse(vertcat(*[x,inf]).is_regular())
    with self.assertRaises(Exception):
      self.assertFalse(vertcat(*[x,x]).is_regular())


  def test_symvar(self):
    a = SX.sym("a")
    b = SX.sym("b")
    c = SX.sym("c")
    e = cos(a*b) + c
    w = symvar(e)
    self.assertEqual(len(w),3)
    if GlobalOptions.getSimplificationOnTheFly():
      self.assertTrue(is_equal(w[0],a))
      self.assertTrue(is_equal(w[1],b))
      self.assertTrue(is_equal(w[2],c))

  def test_poly_coeff(self):
    x =SX.sym("x")
    a= SX.sym("a")
    c=SX.sym("c")
    p=poly_coeff(12*x**4+x**2+a*x+c,x)
    self.assertTrue(is_equal(p[0],12))
    self.assertTrue(is_equal(p[1],0))
    self.assertTrue(is_equal(p[2],1))
    self.assertTrue(is_equal(p[3],a))
    self.assertTrue(is_equal(p[4],c))

    p=poly_coeff((x-a)*(x+a),x)
    self.assertTrue(is_equal(p[0],1))
    self.assertTrue(is_equal(p[1],0))

  def test_poly_roots(self):

    p = SX.sym("[a,b]")
    r = poly_roots(p)

    f = Function("f", [p],[r])
    f_in = [0]*f.n_in()
    f_in[0]=DM([2,7])
    a_ = f_in[0][0]
    b_ = f_in[0][1]
    f_out = f.call(f_in)
    f_out[0]
    self.checkarray(f_out[0],vertcat(*[-b_/a_]))

    p = SX.sym("[a,b]")
    r = poly_roots(vertcat(*[p,0]))

    f = Function("f", [p],[r])
    f_in = [0]*f.n_in();f_in[0]=DM([2,7])
    a_ = f_in[0][0]
    b_ = f_in[0][1]
    f_out = f.call(f_in)
    f_out[0]
    self.checkarray(f_out[0],vertcat(*[-b_/a_,0]))

    p = SX.sym("[a,b,c]")
    r = poly_roots(p)

    f = Function("f", [p],[r])
    f_in = [0]*f.n_in();f_in[0]=DM([1.13,7,3])
    a_ = f_in[0][0]
    b_ = f_in[0][1]
    c_ = f_in[0][2]
    d = b_**2-4*a_*c_
    f_out = f.call(f_in)
    x0 = (-b_-sqrt(d))/2/a_
    x1 = (-b_+sqrt(d))/2/a_
    f_out[0]
    self.checkarray(f_out[0],vertcat(*[x0,x1]))

    p = SX.sym("[a,b,c,d]")
    r = poly_roots(p)

    f = Function("f", [p],[r])
    f_in = [0]*f.n_in();f_in[0]=DM([11,1.3,-1.7,0.1])
    f_out = f.call(f_in)
    f_out[0]
    self.checkarray(f_out[0],DM([0.298028,-0.479787,0.0635774]),digits=5)

    p = SX.sym("[a,b,c,d,e]")
    r = poly_roots(p)

    f = Function("f", [p],[r])
    f_in = [0]*f.n_in();f_in[0]=DM([3,6,-123,  -126,1080])
    f_out = f.call(f_in)
    f_out[0]
    self.checkarray(f_out[0],DM([5,3,-4,-6]),digits=5)

  def test_eig_symbolic(self):
    x = SX.sym("x",2,2)
    f = Function("f", [x],[eig_symbolic(x)])
    f_in = [0]*f.n_in();f_in[0]=DM([[2,0.1],[0.3,0.7]])
    f_out = f.call(f_in)
    self.checkarray(f_out[0],DM([0.67732,2.02268]),digits=5)


    x = SX.sym("x",2)
    f = Function("f", [x],[eig_symbolic(c.diag(x))])
    f_in = [0]*f.n_in();f_in[0]=DM([3,7])
    f_out = f.call(f_in)
    self.checkarray(f_out[0],f_in[0])


    x = SX.sym("x",5)
    f = Function("f", [x],[eig_symbolic(c.diag(x))])
    f_in = [0]*f.n_in();f_in[0]=DM([3,7,2,1,6])
    f_out = f.call(f_in)
    self.checkarray(f_out[0],f_in[0])

    x = SX.sym("x",2,2)
    y = SX.sym("y",2)
    f = Function("f", [x,y],[eig_symbolic(diagcat(*[x,c.diag(y)]))])
    f_in = [0]*f.n_in();f_in[0]=DM([[2,0.1],[0.3,0.7]])
    f_in[1]=[3,7]
    f_out = f.call(f_in)
    self.checkarray(f_out[0],DM([0.67732,2.02268,3,7]),digits=5)

    x = SX.sym("x",3,3)
    x[2,0] = 0
    x[1,0] = 0

    x = sparsify(x)

    e = eig_symbolic(x)

    f = Function("f", [x],[e])
    f_in = [0]*f.n_in();f_in[0]=DM(f.sparsity_in(0),list(range(1,8)))
    f_in[0].print_dense()
    f_out = f.call(f_in)
    self.checkarray(f_out[0],DM([1,-0.29150,10.29150]),digits=5)


    x = SX.sym("x",3,3)
    x[2,0] = 0
    x[1,0] = 0
    x[2,1] = 0

    x = sparsify(x)

    e = eig_symbolic(x)

    f = Function("f", [x],[e])
    f_in = [0]*f.n_in();f_in[0]=DM(f.sparsity_in(0),list(range(1,7)))
    f_in[0].print_dense()
    f_out = f.call(f_in)
    self.checkarray(f_out[0],DM([1,3,6]),digits=5)

    x = SX.sym("x",Sparsity.upper(5))

    f = Function("f", [x],[eig_symbolic(x)])
    fin = DM(x.sparsity(),0)
    fin[Sparsity.diag(5)] = c.diag(list(range(5)))
    self.checkarray(f(fin), DM(list(range(5))))

  def test_jacobian_empty(self):
    x = SX.sym("x",3)

    s = jacobian(DM(0,0),x).shape
    self.assertEqual(s[0],0)
    self.assertEqual(s[1],3)

    s = jacobian(x,SX.sym("x",0,4)).shape
    self.assertEqual(s[0],3)
    self.assertEqual(s[1],0)

  def test_empty_SX(self):
    s = SX([]).shape
    self.assertEqual(s[0],0)
    self.assertEqual(s[1],1)
    x = vertcat(*(SX.sym("x"),SX([])))

    SX(Sparsity(3,3), [])

    SX(Sparsity(3,3), [5])
    with self.assertInException("fully sparse"):
      SX(Sparsity(3,3), [5,4])

  def test_mul_sparsity(self):

    N = 10
    x = SX.sym("x",N,N)
    y = SX.sym("y",N,N)

    x_ = self.randDM(N,N)
    y_ = self.randDM(N,N)

    filt = Sparsity.diag(N)+Sparsity.triplet(N,N,[1],[3])

    f = Function("f", [x,y],[mtimes(x,y)])
    f_in = [0]*f.n_in();f_in[0]=x_
    f_in[1]=y_
    g = Function("g", [x,y],[mac(x,y,SX.zeros(filt))])
    g_in = [0]*g.n_in();g_in[0]=x_
    g_in[1]=y_

    f_out = f.call(f_in)
    g_out = g.call(g_in)

    self.checkarray(DM.ones(filt),DM.ones(g.sparsity_out(0)))

    self.checkarray(f_out[0][filt],g_out[0])

  @skip(platform_arch==32)
  @memory_heavy()
  @unittest.skipIf(sys.version_info >= (3, 0),"pickle is not compatible")
  def test_large_hessian(self):
    import pickle

    A = Sparsity.from_file("../data/apoa1-2.mtx")


    H = DM(A,list(range(A.nnz())))
    H = H + H.T

    H = H[:20000,:20000]

    x = SX.sym("x",H.size1())

    f = Function("f", [x],[mtimes([x.T,H,x])], {'verbose':True})
    H *= 2

    h = hessian_old(f, 0, 0)
    h_out = h.call([0])

    self.assertTrue(h.sparsity_out(0)==H.sparsity())

    self.checkarray(h_out[0].nonzeros(),H.nonzeros())

  def test_mxnulloutput(self):
     a = SX(5,0)
     b = SX.sym("x",2)
     bm = MX.sym("x",2)

     f = Function("f", [b],[a])
     c = f(bm)

     self.assertEqual(c.size1(),5)
     self.assertEqual(c.size2(),0)

     c = f(b)

     self.assertEqual(c.size1(),5)
     self.assertEqual(c.size2(),0)

     a = SX(0,0)

     f = Function("f", [b],[a])

     c = f(bm)

     self.assertEqual(c.size1(),0)
     self.assertEqual(c.size2(),0)

     c = f(b)

     self.assertEqual(c.size1(),0)
     self.assertEqual(c.size2(),0)

  def test_mxnull(self):
     a = SX(5,0)
     b = SX(0,3)

     c = mtimes(a,b)

     self.assertEqual(c.nnz(),0)

     a = SX(5,3)
     b = SX(3,4)

     c = mtimes(a,b)

     self.assertEqual(c.nnz(),0)

  def  test_mxnullop(self):
    c = SX(0,0)
    x = SX.sym("x",2,3)

    # https://github.com/casadi/casadi/issues/2628
    if swig4:
      with self.assertRaises(TypeError):
        d = x + c
    else:
      with self.assertRaises(RuntimeError):
        d = x + c

    if swig4:
      with self.assertRaises(TypeError):
        d = x / c
    else:
      with self.assertRaises(RuntimeError):
        d = x / c

  def test_copysign(self):
    x = SX.sym("x")
    y = SX.sym("y")
    z = copysign(x,y)

    f = Function("f", [x,y],[z])

    f_in = [0]*f.n_in();f_in[0]=2
    f_in[1]=0.5
    f_out = f.call(f_in)
    self.checkarray(f_out[0],DM([2]))

    f_in = [0]*f.n_in();f_in[0]=2
    f_in[1]=-0.5
    f_out = f.call(f_in)
    self.checkarray(f_out[0],DM([-2]))

    f_in = [0]*f.n_in();f_in[0]=-2
    f_in[1]=0.5
    f_out = f.call(f_in)
    self.checkarray(f_out[0],DM([2]))

    f_in = [0]*f.n_in();f_in[0]=-2
    f_in[1]=-0.5
    f_out = f.call(f_in)
    self.checkarray(f_out[0],DM([-2]))

    f_in = [0]*f.n_in();f_in[0]=2
    f_in[1]=0
    f_out = f.call(f_in)
    self.checkarray(f_out[0],DM([2]))

    J = jacobian_old(f, 0, 0)

    J_in = [0]*J.n_in();J_in[0]=2
    J_in[1]=0.5
    J_out = J.call(J_in)
    self.checkarray(J_out[0],DM([1]))

    J_in = [0]*J.n_in();J_in[0]=2
    J_in[1]=-0.5
    J_out = J.call(J_in)
    self.checkarray(J_out[0],DM([-1]))

    J_in = [0]*J.n_in();J_in[0]=-2
    J_in[1]=0.5
    J_out = J.call(J_in)
    self.checkarray(J_out[0],DM([1]))

    J_in = [0]*J.n_in();J_in[0]=-2
    J_in[1]=-0.5
    J_out = J.call(J_in)
    self.checkarray(J_out[0],DM([-1]))

    J_in = [0]*J.n_in();J_in[0]=2
    J_in[1]=0
    J_out = J.call(J_in)
    self.checkarray(J_out[0],DM([1]))

    J = jacobian_old(f, 1, 0)

    J_in = [0]*J.n_in();J_in[0]=2
    J_in[1]=0.5
    J_out = J.call(J_in)
    self.checkarray(J_out[0],DM([0]))

    J_in = [0]*J.n_in();J_in[0]=2
    J_in[1]=-0.5
    J_out = J.call(J_in)
    self.checkarray(J_out[0],DM([0]))

    J_in = [0]*J.n_in();J_in[0]=-2
    J_in[1]=0.5
    J_out = J.call(J_in)
    self.checkarray(J_out[0],DM([0]))

    J_in = [0]*J.n_in();J_in[0]=-2
    J_in[1]=-0.5
    J_out = J.call(J_in)
    self.checkarray(J_out[0],DM([0]))

    J_in = [0]*J.n_in();J_in[0]=2
    J_in[1]=0
    J_out = J.call(J_in)
    self.checkarray(J_out[0],DM([0]))

  def test_depends_on(self):
    a = SX.sym("a")
    b = SX.sym("b")

    self.assertTrue(depends_on(a**2,a))
    self.assertTrue(depends_on(a,a))
    self.assertFalse(depends_on(0,a))
    self.assertTrue(depends_on(a**2,vertcat(*[a,b])))
    self.assertTrue(depends_on(a,vertcat(*[a,b])))
    self.assertFalse(depends_on(0,vertcat(*[a,b])))
    self.assertTrue(depends_on(b**2,vertcat(*[a,b])))
    self.assertTrue(depends_on(b,vertcat(*[a,b])))
    self.assertTrue(depends_on(a**2+b**2,vertcat(*[a,b])))
    self.assertTrue(depends_on(a+b,vertcat(*[a,b])))
    self.assertTrue(depends_on(vertcat(*[0,a]),a))
    self.assertTrue(depends_on(vertcat(*[a,0]),a))
    self.assertTrue(depends_on(vertcat(*[a**2,b**2]),vertcat(*[a,b])))
    self.assertTrue(depends_on(vertcat(*[a,0]),vertcat(*[a,b])))
    self.assertTrue(depends_on(vertcat(*[0,b]),vertcat(*[a,b])))
    self.assertTrue(depends_on(vertcat(*[b,0]),vertcat(*[a,b])))
    self.assertFalse(depends_on(vertcat(*[0,0]),vertcat(*[a,b])))

  @requires("is_smooth")
  def test_is_smooth(self):
    x = SX.sym("a",2,2)
    import warnings
    with warnings.catch_warnings():
      warnings.simplefilter("error",DeprecationWarning)
      with self.assertRaises(Exception):
        is_smooth(x)
      warnings.simplefilter("ignore")
      is_smooth(x)

  def test_which_depends(self):
    for X in [SX,MX]:
      x = X.sym("x")
      y = X.sym("y")

      p = X.sym("p")

      e = vertcat(0,x,y,p,2*p**3,x*y,x*p,sin(x),cos(y),sqrt(x+y),p*p*x,x*y*p)

      self.checkarray(which_depends(e, vertcat(x,y),2,True),[0, 0, 0, 0,0, 1, 0, 1, 1, 1, 0, 1])
      self.checkarray(which_depends(e, vertcat(x,y),1,True),[0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1])

      z =X.sym("z")
      e = vertcat(x*p,x+y)
      self.checkarray(which_depends(e, vertcat(x,y,p,z),2,False),[True, False, True, False])
      self.checkarray(which_depends(e, vertcat(x,y,p,z),1,False),[True, True, True, False])

      e = vertcat(x*p,x+z*y)
      self.checkarray(which_depends(e, vertcat(x,y,p),2,False),[True, False, True])
      self.checkarray(which_depends(e, vertcat(x,y,p),1,False),[True, True, True])

      e = vertcat(x*p,x+z*y)
      self.checkarray(which_depends(e, vertcat(x,y,p,z),2,False),[True, True, True, True])
      self.checkarray(which_depends(e, vertcat(x,y,p,z),1,False),[True, True, True, True])

      e = vertcat(sin(x+y)+p)
      self.checkarray(which_depends(e, vertcat(x,y,p,z),2,False),[True, True, False, False])
      self.checkarray(which_depends(e, vertcat(x,y,p,z),1,False),[True, True, True, False])

      e = vertcat(sin(x)*p**2,y**2)
      #self.checkarray(which_depends(e, vertcat(x,y,p),3,True),[True, False])
      #self.checkarray(which_depends(e, vertcat(x,y,p),3,False),[True, False, True])
      self.checkarray(which_depends(e, vertcat(x,y,p),2,True),[True, True])
      self.checkarray(which_depends(e, vertcat(x,y,p),2,False),[True, True, True])

      e = vertcat(x**2*p,y)
      #self.checkarray(which_depends(e, vertcat(x,y,p),3,True),[True, False])
      #self.checkarray(which_depends(e, vertcat(x,y,p),3,False),[True, False, False])

      self.checkarray(which_depends(e, vertcat(x,y,p),2,True),[True, False])
      self.checkarray(which_depends(e, vertcat(x,y,p),2,False),[True, False, True])

  def test_if_else_zero_sens(self):

    for X in [SX]:
      x=X.sym('x')


      a = 1+3*x+sqrt(3*x)*x+7*x
      b = 1+2*x+sin(2*x)*x +x
      z = if_else(x>0,a,b)*x

      f = Function("f",[x],[z,jacobian(z,x)])
      fa = Function("f",[x],[a*x,jacobian(a*x,x)])
      fb = Function("f",[x],[b*x,jacobian(b*x,x)])

      for i,j in zip(f([3]),fa([3])):
        self.checkarray(i,j)

      for i,j in zip(f([-3]),fb([-3])):
        self.checkarray(i,j)


      f = Function("f",[x],[z])
      fa = Function("f",[x],[a*x])
      fb = Function("f",[x],[b*x])

      self.checkfunction(f,fa,inputs=[3])
      self.checkfunction(f,fb,inputs=[-3],evals=1)

  def test_pw_const(self):
      t= SX.sym("t")

      e = pw_const(t, [0,2,3],[7,1,3,5])

      E = Function("E",[t],[e])

      self.checkarray(E(-2),7)
      self.checkarray(E(-1),7)
      self.checkarray(E(0),1)
      self.checkarray(E(1),1)
      self.checkarray(E(1.9999),1)
      self.checkarray(E(2),3)
      self.checkarray(E(2.5),3)
      self.checkarray(E(3),5)
      self.checkarray(E(10),5)

  def test_pw_lin(self):
      t= SX.sym("t")

      e = pw_lin(t, [0,2,3,5], [7,1,3,2])

      E = Function("E",[t],[e])

      self.checkarray(E(-2),13)
      self.checkarray(E(-1),10)
      self.checkarray(E(0),7)
      self.checkarray(E(1),4)
      self.checkarray(E(2),1)
      self.checkarray(E(2.5),2)
      self.checkarray(E(3),3)
      self.checkarray(E(4),2.5)
      self.checkarray(E(5),2)
      self.checkarray(E(7),1)

  def test_numpy_error(self):
      x = SX.sym("x",3)
      with self.assertInException("Use an equivalent CasADi function"):
        np.linalg.norm(x)


  def test_quadratic(self):
    for X in [SX,MX]:
      x = X.sym("x")
      p = X.sym("p")
      y = X.sym("y")

      self.assertFalse(is_quadratic(sin(x),x))
      self.assertFalse(is_quadratic(x**3,x))
      self.assertTrue(is_quadratic(x**2,x))
      self.assertTrue(is_quadratic(4*x,x))
      self.assertTrue(is_quadratic(5,x))

      self.assertFalse(is_quadratic(sin(x)*p**4,x))
      self.assertFalse(is_quadratic(x**3*p**4,x))
      self.assertTrue(is_quadratic(x**2*p**4,x))
      self.assertTrue(is_quadratic(x*p**4,x))
      self.assertTrue(is_quadratic(5*p**4,x))

      self.assertFalse(is_linear(sin(x),x))
      self.assertFalse(is_linear(x**3,x))
      self.assertFalse(is_linear(x**2,x))
      self.assertTrue(is_linear(3*x,x))
      self.assertTrue(is_linear(5,x))

      self.assertFalse(is_linear(sin(x)*p**4,x))
      self.assertFalse(is_linear(x**3*p**4,x))
      self.assertFalse(is_linear(x**2*p**4,x))
      self.assertTrue(is_linear(x*p**4,x))
      self.assertTrue(is_linear(5*p**4,x))



      z = x**2+3*y**2 + 0.5*x*y + 7*x + 6*y+7
      [A,b,c] = quadratic_coeff(z,vertcat(x,y))

      with self.assertInException("non-quadratic"):
        [A,b,c] = quadratic_coeff(x**2+3*y**2 + 0.5*x*y + 7*x + 6*y+7+sin(x),vertcat(x,y))

      with self.assertInException("scalar"):
        [A,b,c] = quadratic_coeff(vertcat(x,y),x)

      z = x**2+3*y**2 + 0.5*x*y -p*y + 7*x + 6*y+7
      [A,b,c] = quadratic_coeff(z,vertcat(x,y))

      xy = vertcat(x,y)

      e = 0.5*bilin(A,xy,xy)+dot(b,xy)+c

      f = Function('f',[xy,p],[z])
      f2 = Function('f',[xy,p],[e])
      self.checkfunction(f,f2,inputs=[1.1,1.3])


      with self.assertInException("non-linear"):
        [A,b] = linear_coeff(x**2+3*y**2 + 0.5*x*y + 7*x + 6*y+7,vertcat(x,y))

      with self.assertInException("vector"):
        [A,b] = linear_coeff(blockcat([[x,y],[y,x]]),x)

      z = vertcat(7*x + 6*y+7 ,5 -p*y )
      [A,b] = linear_coeff(z,xy)

      e = mtimes(A,xy)+b

      f = Function('f',[xy,p],[z])
      f2 = Function('f',[xy,p],[e])
      self.checkfunction(f,f2,inputs=[1.1,1.3])

  def test_evalf(self):
    x = SX.sym("x")

    y = SX(5)

    self.checkarray(evalf(y),5)
    with self.assertInException("since variables [x] are free"):
      evalf(x)
  

  def test_output_sx(self):
  
    x = MX.sym("x")
    y = MX.sym("y")
    z = MX.sym("z")
    w = mtimes(sqrt(z),sin(x*y))

    f = Function('f',[x,y,z],[x*y*z,w])
    print(f)

    x = SX.sym("x")
    y = SX.sym("y")
    z = SX.sym("z")
    
    args = [x/y,10*z,y*z]

    v = f.call(args,False,True)

    output_0 = v[0]
    output_1 = v[1]    


    self.assertEqual(output_0.dep(0).element_hash(),output_1.dep(0).element_hash())
    call_node = output_0.dep(0)
    
    print("here")
    
    v = None
    
    # Check that get_output is cached
    output_1b = call_node.get_output(1)
    self.assertEqual(output_1.element_hash(),output_1b.element_hash())
    
    h1 = output_1.element_hash()
    
    output_1 = None
    output_1b = None
    
    # Without output_0 gone, output_1b seems to get consistently reconstructed into the freshly deleted memory slot
    output_0 = None

    output_1b = call_node.get_output(1)
    
    # This test is too fragile
    #self.assertNotEqual(h1,output_1b.element_hash())


  def test_call_fun(self):

    A = sparsify(DM([[1,0,1],[0,0,6],[0,8,9]]))

    x = MX.sym("x",3)
    y = MX.sym("y")
    z = MX.sym("z")
    w = mtimes(A*sqrt(z),sin(x*y))

    f = Function('f',[x,y,z],[(x*y*z)[:2],w])
    print(f)

    x = SX.sym("x",3)
    y = SX.sym("y")
    z = SX.sym("z")
    
    args = [x/y,10*z,y*z]

    v = f(*args)
    v2 = f.call(args,False,True)
    print(v2)
    print(vcat(v2))
    

    self.assertTrue(v2[0][0].is_output())
    self.assertFalse(y.is_output())
    k = 0
    for i in range(f.n_out()):
        for j in range(f.nnz_out(i)):
            self.assertEqual(v2[i][j].which_output(),k)
            k+=1
    callnode = v2[0][0].dep(0)
    self.assertTrue(callnode.is_call())
    self.assertFalse(y.is_call())
    self.assertTrue(callnode.has_output())
    self.assertFalse(y.has_output())
    self.assertTrue(callnode.which_function().__hash__()==f.__hash__())
    
    self.assertTrue(callnode.n_dep()==5)
    callnode.dep(4)
    
    e = callnode.get_output(3)
    print(e,hash(e))
    
    ee = callnode.get_output(3)
    print(ee,hash(e))
    
    
    self.assertTrue("{3}" in str(callnode.get_output(3)))
    
    print(cos(mtimes(v2[1],v2[0].T)/y).shape)
    F2 = Function('F',[x,y,z],[sin(v2[0]*y-args[-1]),cos(mtimes(v2[1],v2[0].T)/y)])
    
    with self.assertOutput("f:(i0[3],i1,i2)->(o0[2],o1[3])",[]):
        print(f)
    with self.assertOutput("[[@2,@3],[@4,@5,@7]] = f([@2,@3,@4],@5,@6);",[]):
        F2.disp(True)

    F1 = Function('F',[x,y,z],[sin(v[0]*y-args[-1]),cos(mtimes(v[1],v[0].T)/y)])    
    for ad_weight in [True,False]:
        for ad_weight_sp in [True,False]:
            F2 = Function('F',[x,y,z],[sin(v2[0]*y-args[-1]),cos(mtimes(v2[1],v2[0].T)/y)],{"ad_weight_sp":ad_weight_sp,"ad_weight":ad_weight})
                    
            self.checkfunction(F1,F2,inputs=[[0.1,1.7,2.3],1.13,0.11])

            self.check_serialize(F2,inputs=[[0.1,1.7,2.3],1.13,0.11])
            for avoid_stack in [True,False]:
                self.check_codegen(F2,inputs=[[0.1,1.7,2.3],1.13,0.11],opts={"avoid_stack":avoid_stack})
    
    instr = F2.instructions_sx()
    check = False
    for k in range(F2.n_instructions()):
        op = F2.instruction_id(k)
        if op==OP_CALL:
            check = True
            self.assertTrue(instr[k].is_call())
            self.assertEqual(F2.instruction_output(k),[2, 3, 4, 5, 7])
            self.assertEqual(F2.instruction_input(k),[2, 3, 4, 5, 6])

    self.assertTrue(check)

    F2 = Function('F',[x,y,z],[sin(v2[0]*y-args[-1]),cos(mtimes(v2[1],v2[0].T)/y)],{"cse":True})
    self.checkfunction(F1,F2,inputs=[[0.1,1.7,2.3],1.13,0.11])
    
    res = F2.find_functions()
    self.assertTrue(len(res)==1)
    self.assertTrue(res[0].__hash__()==f.__hash__())

    F2 = Function('F',[x,y,z],F2(x,y,z))

    self.checkfunction(F1,F2,inputs=[[0.1,1.7,2.3],1.13,0.11])

    F1 = Function('F',[x,y,z],F1(2*x,3*y,4*z))
    F2 = Function('F',[x,y,z],F2(2*x,3*y,4*z))

    self.checkfunction(F1,F2,inputs=[[0.1,1.7,2.3],1.13,0.11])

    F1 = Function('F',[x,y,z],F1(x,3*y,4*z))
    F2 = Function('F',[x,y,z],F2(x,3*y,4*z))

    self.checkfunction(F1,F2,inputs=[[0.1,1.7,2.3],1.13,0.11])
    
    
    F2 = Function('F',[x,y,z],[sin(v2[0]*y-args[-1]),cos(mtimes(v2[1],v2[0].T)[1]/y)])
    with self.assertOutput("[[@2,@3],[NULL,@4,NULL]] = f([@2,@3,@4],@5,@6);",[]):
        F2.disp(True)

    F1 = Function('F',[x,y,z],[sin(v[0]*y-args[-1]),cos(mtimes(v[1],v[0].T)[1]/y)])    
    for ad_weight in [True,False]:
        for ad_weight_sp in [True,False]:
            F2 = Function('F',[x,y,z],[sin(v2[0]*y-args[-1]),cos(mtimes(v2[1],v2[0].T)[1]/y)],{"ad_weight_sp":ad_weight_sp,"ad_weight":ad_weight})
                    
            self.checkfunction(F1,F2,inputs=[[0.1,1.7,2.3],1.13,0.11])

            self.check_serialize(F2,inputs=[[0.1,1.7,2.3],1.13,0.11])
            for avoid_stack in [True,False]:
                self.check_codegen(F2,inputs=[[0.1,1.7,2.3],1.13,0.11],opts={"avoid_stack":avoid_stack})
    
    # multiple instances in one graph
 
  def test_call_fun2(self):
  
    print("test_call_fun2")

    for X in [SX,MX]:
    
        for never_inline in [True, False]:
            print(X, never_inline)
            A = sparsify(DM([[1,0,1],[0,0,6],[0,8,9]]))

            x = X.sym("x",3)
            y = X.sym("y")
            z = X.sym("z")
            w = mtimes(A*sqrt(z),sin(x*y))
            
            f = Function('fun',[x,y,z],[(x*y*z)[:2],w],{"never_inline":never_inline})

            x = SX.sym("x",3)
            y = SX.sym("y")
            z = SX.sym("z")
            
            args = [x/y,10*z,y*z]
            res = f(*args)
            print(res)
            self.assertEqual('fun' in str(res), never_inline)
        
  def test_call_fun_nominal_out_deriv(self):
    x = SX.sym("x",2)
    p = SX.sym("p",2)
    rf = rootfinder('rf',"newton",{'x':x,"g":vertcat(sin(x[0])-p[0],sin(x[0]+x[1])-p[1]*p[0]),"p":p})
    
    f = Function("f",[p],[exp(rf(p=p**2)["x"])])
    P = MX.sym("p",2)
    fref = Function("f",[P],[exp(rf(p=P**2)["x"])])
    
    self.checkfunction(f,fref,inputs=[[0.1,0.2]])
  
  def test_call_fun_copy_elision(self):
    old = GlobalOptions.getCopyElisionMinSize()
    GlobalOptions.setCopyElisionMinSize(0)
    
    mX = MX.sym("A",3,6)
    mY = MX.sym("Y")
    
    X = SX.sym("A",3,6)
    Y = SX.sym("Y")
    
    DM.rng(1)
    
    g = Function('g',[X,Y],[X,Y])
    gref = Function('g',[mX,mY],[mX,mY])
    inputs = [DM.rand(3,3),DM.rand(1)]
    self.checkfunction(g,gref,inputs=inputs)
    for avoid_stack in [True,False]:
        self.check_codegen(g,inputs=inputs,opts={"avoid_stack":avoid_stack})
    g_roundtrip = Function.deserialize(g.serialize())
    self.checkfunction(g,g_roundtrip,inputs=inputs)
    for avoid_stack in [True,False]:
        self.check_codegen(g_roundtrip,inputs=inputs,opts={"avoid_stack":avoid_stack})

    X = MX.sym("A",3,3)
    Y = MX.sym("Y")

    f = Function('f',[X,Y],[sumsqr(X)*Y],{"never_inline":True})
    fref = Function('f',[X,Y],[sumsqr(X)*Y])
    X = SX.sym("A",3,6)
    Y = SX.sym("Y")
    
    DM.rng(1)
    
    g = Function('g',[X,Y],[f(X[:,3:],sin(Y)),X[1,3],3*X[2,4]])
    gref = Function('g',[X,Y],[fref(X[:,3:],sin(Y)),X[1,3],3*X[2,4]])
    inputs = [DM.rand(3,3),DM.rand(1)]
    self.checkfunction(g,gref,inputs=inputs)
    for avoid_stack in [True,False]:
        self.check_codegen(g,inputs=inputs,opts={"avoid_stack":avoid_stack})
    g_roundtrip = Function.deserialize(g.serialize())
    self.checkfunction(g,g_roundtrip,inputs=inputs)
    for avoid_stack in [True,False]:
        self.check_codegen(g_roundtrip,inputs=inputs,opts={"avoid_stack":avoid_stack})

    X = MX.sym("A",4)
    Y = MX.sym("Y")

    f = Function('f',[X,Y],[sumsqr(X)*Y],{"never_inline":True})
    fref = Function('f',[X,Y],[sumsqr(X)*Y])
    X = SX.sym("A",6)
    Y = SX.sym("Y")
    
    DM.rng(1)
    
    g = Function('g',[X,Y],[f(vertcat(2,X[:3]),sin(Y))])
    gref = Function('g',[X,Y],[fref(vertcat(2,X[:3]),sin(Y))])
    inputs = [DM.rand(6),DM.rand(1)]
    self.checkfunction(g,gref,inputs=inputs)
    for avoid_stack in [True,False]:
        self.check_codegen(g,inputs=inputs,opts={"avoid_stack":avoid_stack})
    g_roundtrip = Function.deserialize(g.serialize())
    self.checkfunction(g,g_roundtrip,inputs=inputs)
    for avoid_stack in [True,False]:
        self.check_codegen(g_roundtrip,inputs=inputs,opts={"avoid_stack":avoid_stack})
        
    g = Function('g',[X,Y],[f(vertcat(2,X[:3]),sin(Y)),3*X])
    gref = Function('g',[X,Y],[fref(vertcat(2,X[:3]),sin(Y)),3*X])
    inputs = [DM.rand(6),DM.rand(1)]
    self.checkfunction(g,gref,inputs=inputs)
    for avoid_stack in [True,False]:
        self.check_codegen(g,inputs=inputs,opts={"avoid_stack":avoid_stack})
    g_roundtrip = Function.deserialize(g.serialize())
    self.checkfunction(g,g_roundtrip,inputs=inputs)
    for avoid_stack in [True,False]:
        self.check_codegen(g_roundtrip,inputs=inputs,opts={"avoid_stack":avoid_stack})
        
    GlobalOptions.setCopyElisionMinSize(old)

  def test_ufunc(self):
    y = np.sin(casadi.SX.sym('x'))

  def test_mmin(self):
      x = SX.sym("X",2)
      f0 = Function("f",[x],[(x[0]+x[1])/2])
      f1 = Function("f",[x],[mmin(x)])
      f2 = Function("f",[x],[fmin(x[0],x[1])])
      self.checkfunction(f1,f2,inputs=[[0.2,0.3]])
      self.checkfunction(f1,f2,inputs=[[2,2]])
      self.checkfunction(f1,f0,inputs=[[2,2]])
      f1 = Function("f",[x],[mmax(x)])
      f2 = Function("f",[x],[fmax(x[0],x[1])])
      self.checkfunction(f1,f2,inputs=[[0.2,0.3]])
      self.checkfunction(f1,f2,inputs=[[2,2]])
      self.checkfunction(f1,f0,inputs=[[2,2]])

      x = SX.sym("X")
      f0 = Function("f",[x],[x])
      f1 = Function("f",[x],[fmin(x,x)])
      self.checkfunction(f1,f0,inputs=[[1]])
      f1 = Function("f",[x],[fmax(x,x)])
      self.checkfunction(f1,f0,inputs=[[1]])

  def test_empty_broadcast(self):
    for nc in [0,2]:
      res = atan2(SX.sym("c",nc,1),SX.sym("t",nc,3))

      self.assertEqual(res.shape[0],nc)
      self.assertEqual(res.shape[1],3)
  
      with self.assertInException("Dimension mismatch"):
        res = atan2(MX.sym("c",nc,2),MX.sym("t",nc,3))

      res = atan2(MX.sym("c",nc,2),MX.sym("t",nc,4))
      self.assertEqual(res.shape[0],nc)
      self.assertEqual(res.shape[1],4)

      res = atan2(MX.sym("c",nc,4),MX.sym("t",nc,2))
      self.assertEqual(res.shape[0],nc)
      self.assertEqual(res.shape[1],4)

  def test_logsumexp(self):
    x = SX.sym("x",3)

    f_ref = Function("f_ref",[x],[log(exp(x[0])+exp(x[1])+exp(x[2]))])
    f = Function("f",[x],[logsumexp(x)])

    self.checkfunction(f,f_ref,inputs=[vertcat(1.1,1.3,1.7)])
    self.check_codegen(f,inputs=[vertcat(1.1,1.3,1.7)])
    self.checkfunction(f,f_ref,inputs=[vertcat(1.1,1.3,1.3)])
    self.checkfunction(f,f_ref,inputs=[vertcat(1.3,1.3,1.3)])

    self.checkarray(logsumexp(vertcat(1.3,1.3,1.3)),f_ref(vertcat(1.3,1.3,1.3)))
    self.checkarray(logsumexp(vertcat(1.1,1.3,1.3)),f_ref(vertcat(1.1,1.3,1.3)))
    self.checkarray(logsumexp(vertcat(1.1,1.3,1.7)),f_ref(vertcat(1.1,1.3,1.7)))

    # Avoid overflow
    res = f(vertcat(100,1000,10000))
    self.checkarray(res,10000)

    self.checkarray(logsumexp(vertcat(100,1000,10000)),f(vertcat(100,1000,10000)))

  def test_extract_parametric_call_sx(self):
    x = MX.sym("x")
    y = MX.sym("y")
    z = MX.sym("z")
    
    f = Function("f",[x,y,z],[x*y,y*z,x*z],{"never_inline":True})
    
    x = SX.sym("x")
    p = SX.sym("p")
    
    [expr1,expr2,expr3] = f(1,sin(p),sqrt(p))
    
    expr = x*(expr1+expr3)
    
    expr_ret,symbols,parametric = extract_parametric(expr,p)
    
    print("expr_ret",expr_ret)
    print("symbols",symbols)
    print("parametric",parametric)
    
    self.assertTrue("x*e_0" in str(expr_ret))
    
    print(expr_ret,symbols,parametric)
    
    self.assertFalse(depends_on(expr_ret,p))
        
    expr_recreated = substitute([expr_ret],symbols,parametric)[0]
    
    print(expr_recreated-expr)
    
    self.assertTrue(cse(expr_recreated-expr).is_zero())


    [expr1,expr2,expr3] = f(1,sin(p),x**2)
    
    expr = expr1+expr3
    

    
    expr_ret,symbols,parametric = extract_parametric(expr,p)
    
    self.assertFalse(depends_on(expr_ret,p))
        
    expr_recreated = substitute([expr_ret],symbols,parametric)[0]
    
    f = Function('f',[x,p],[expr_recreated])
    f.generate('f1.c')
    f = Function('f',[x,p],[expr])
    f.generate('f2.c')
    cse(expr_recreated)
    self.assertTrue(cse(expr_recreated-expr).is_zero())
    
  def test_check_recursion(self):
    x = SX.sym("x")


    f = Function("f",[x],[x**2],{"always_inline":True})

    y = MX.sym("y")
    
    f(y)

  def test_sx_eval_mx(self):
    n = 2
    y = MX.sym("y",Sparsity.lower(n))

    g = Function('g',[y],[sin(y)],{"never_inline":True})

    x = SX.sym("x",Sparsity.lower(n))

    f = Function("f",[x],[g(x**2)-2])


    X = MX.sym("X",Sparsity.upper(n))

    F1 = Function("F1",[X],f.call([X],True))

    DM.rng(1)
    A = DM.rand(n,n)

    F2 = Function("F2",[X],f.call([X]))

    self.checkfunction_light(F1,F2,inputs=[A])

  def test_contains(self):
    x = SX.sym("x")
    y = SX.sym("y")
    z = SX.sym("z")
    
    e = y*z
    
    self.assertTrue(contains([x,y,z],x))
    self.assertFalse(contains([x,y],z))
    self.assertTrue(contains_any([x,y],[y,z]))
    self.assertFalse(contains_all([x,y],[y,z]))
    self.assertTrue(contains_any([x,y],[x,y]))
    self.assertTrue(contains_all([x,y],[x,y]))
    self.assertTrue(contains([e,x],e))
    
    with self.assertInException("Can only convert 1-by-1 matrices to scalars"):
        contains([vertcat(x,y)],x)
    
  def test_pow(self):
    x = SX.sym("x")
    y = SX(4,1)
    self.assertEqual((y**0).nnz(),4)
    y[1] = x
    self.assertEqual((y**0).nnz(),4)
    y[1] = 0
    self.assertEqual((y**0).nnz(),4)
    y[1] = 1
    self.assertEqual((y**0).nnz(),4)

  def test_linearize(self):
    x = SX.sym("x")
    y = sin(x)
    x0 = 0.2
    print(linearize(sin(x),x,x0))
    print(sin(x0)+cos(x0)*(x-x0))
    F = Function('F',[x],[y])
    Ff = F.forward(1)
    n = F(x0)
    print(n + Ff(x0,n,x-x0))
    print(taylor(y,x,x0))

if __name__ == '__main__':
    unittest.main()



import os
import re
import baseconv # pip install python-baseconv

next_identifier = 0

try:
    with open("next_identifier.txt","r") as inp:
        next_identifier = int(inp.read())
except:
    pass
    
print("next_identifier",next_identifier)
    
def id2str(a):
    return baseconv.base36.encode(a)
def str2id(a):
    return int(baseconv.base36.decode(a))

re_comments = re.compile(r"^ */\*\* (.*?)\*/$", re.MULTILINE | re.DOTALL)
identifiers = set()
location = {}

def is_blank(line):
    line = line.strip()
    return line=="*" or line==""


for root, dirs, files in os.walk("../casadi"):
   for name in files:
      if "runtime" in root: continue
      # "core" not in root and  "core" not in root : continue
      if name.endswith(".hpp"):
          file = os.path.join(root, name)
          with open(file,"r") as inp:
            data = inp.read()
            orig = data
            def process(match):
                global next_identifier
                if ("core" in root and ("\\brief" in match.group(0))) or "\\defgroup main_" in match.group(0) or "\\defgroup plugin_" in match.group(0):
                 if "\\identifier" in match.group(0):
                    identifier = str2id(re.search(r"\\identifier\{(.*?)\}",match.group(0)).group(1))
                    line = data[:match.start()].count("\n")+1
                    loc = file + ":" + str(line) + "\n" + match.group(0)
                    if identifier in identifiers:
                        raise Exception(f"Duplicate identifiers:\n{loc}\n{location[identifier]}")
                    identifiers.add(identifier)
                    location[identifier] = loc
                    
                    # Make sure there is a blank line after \brief when multi-line
                    # If there is no such newline, doxygen xml lumps brief and subsequent lines into a single para
                    if "\\brief" in match.group(0):
                        if "\n" in match.group(0):
                            split = match.group(0).split("\n")
                            if not is_blank(split[1]):
                                split = [split[0]] + [""]+split[1:]
                                return "\n".join(split)
                    # Make sure a defgroup starts with \par (needed for doxygeb 1.8.17)
                    if "defgroup main_" in match.group(0) or "defgroup plugin_" in match.group(0):
                        split = match.group(0).split("\n")
                        if "\\par" not in split[1]:
                            split = [split[0]] + [(" "*match.group(0).index("\\defgroup"))+"\\par"]+split[1:]
                            return "\n".join(split)
                        
                    # Make sure there is a blank line before \identifier
                    split = match.group(0).split("\n")
                    identifier_index = [i for i,e in enumerate(split) if e.strip().startswith("\\identifier")][0]
                    if not is_blank(split[identifier_index-1]):
                        split = split[:identifier_index] + [""] + split[identifier_index:]
                        return "\n".join(split)
                 else:
                    m = match.group(0).rstrip()
                    if "\\brief" in m:
                        ident = m.find("\\brief")*" "
                    else:
                        ident = m.find("\\defgroup")*" "
                    next_identifier+=1
                    return m[:-2].rstrip() + "\n\n" + ident+ "\\identifier{" + id2str(next_identifier) + "}" + " */"
                return match.group(0)
            data = re.sub(re_comments, process, data)
          if data!=orig:
            print("Updating", file)
            with open(file,"w") as outp:
              outp.write(data)
            #print(data)

print("next_identifier",next_identifier)
with open("next_identifier.txt","w") as outp:
    outp.write(str(next_identifier))
